#!/usr/bin/python3


''' Import library for running bash scripts and passing bash arguments. '''

import os
from mupy.mucli.selection_branch import SelectionBranch
from mupy.mucli.input_branch import InputBranch
from mupy.lib.CUBX0006.cubx0006 import CUBX0006, CUBX0006_encoding
from mupy.hardware import Hardware
from pathlib import Path

def encoding_session():

    enter_a_number_branch = InputBranch("Enter a number from 1 to 10")
    CUBX0177_BP_input = SelectionBranch("Select adaptive geometry.")
    CUBX0177_BP_input.options.append(["1", "BEAR", "Bearing Array", exit])
    CUBX0177_BP_input.options.append(["2", "NOSF", "Binding Array", exit])
    CUBX0177_BP_input.options.append(["3", "AD", "Binding Array", exit])
    CUBX0177_BP_input.options.append(["b", "Back", "Navigate to previous menu.", exit])

    CUBX0006_input = SelectionBranch("Select type:")
    CUBX0006_input.options.append(["1", "BP", "Box Panel", CUBX0177_BP_input.run])
    
    CUBX0012_input = SelectionBranch("Select type:")
    CUBX0012_input.options.append(["1", "BP", "Box Panel", CUBX0177_BP_input.run])
    CUBX0012_input.options.append(["2", "SP", "Simple Panel", exit])
    CUBX0012_input.options.append(["3", "AX", "Axle", exit])
    CUBX0012_input.options.append(["4", "AD", "Axle Adapter", exit])
    CUBX0012_input.options.append(["b", "Back", "Navigate to previous menu.", exit])

    hardware_generation_code_input = SelectionBranch("Enter hardware code...")

    CUBX00006_encoding = CUBX0006_encoding()
    family_selection_branch = SelectionBranch("Select hardware family")
    family_selection_branch.options.append(["1", "CUBX0006", "The most basic cubic family. This family features a single block type 'BLK' and takes in parameters which indicate length, width and height of a solid block.", CUBX00006_encoding.encode_session ])
    
    family_selection_branch.options.append(["2", "CUBX0012", "An original cubic prototype family featuring a trapezoidal type profile with bolt-snap style binding.", CUBX0012_input.run])
    family_selection_branch.options.append(["3", "CUBX0177", "A versatile cubic family featuring parts made from arrays of 'unit blocks' with a joinery-bolt-custom style binding.", exit])
    family_selection_branch.options.append(["4", "CYLX0008", "A hardware family dedicated to defining pipes.", enter_a_number_branch.run])
    family_selection_branch.options.append(["5", "BOLTX002", "Hardware family dedicated to nuts, bolts and screws.", exit])
    family_selection_branch.options.append(["6", "GEAR0010", "A gear family used for uses in power transmission.", exit])
    family_selection_branch.options.append(["b", "Back", "Navigate to previous menu.", encoding_session])

    family_selection_branch.run()
    
mu_symbol = '\u00B5'
print("\n"+mu_symbol+"term ~ manufacturing utility terminal\n") 
relative_workspace_path=""
while len(relative_workspace_path) == 0:
    relative_workspace_path = input(mu_symbol+":! To proceed you must specify an existing workspace directory or create a new workspace directory relative to home path (:~/$) \n\n    > ")
    relative_workspace_path.replace(" ", "_")
absolute_workspace_path = str(Path.home()) + "/" + relative_workspace_path
if not os.path.exists(absolute_workspace_path):
    os.makedirs(absolute_workspace_path)
    print("\n"+mu_symbol+":! Setting up workspace directory : "+absolute_workspace_path)
else:
    print("\n"+mu_symbol+":! Utilizing workspace directory : "+absolute_workspace_path)
print(""+mu_symbol+":! Press 'Enter' to begin an encoding session or specify a hardware code and press 'Enter'\n") 
while True: # Main loop
    user_input = input(mu_symbol+":# ") 
    if user_input == "exit" or user_input == "close" or user_input == "done" or user_input == "leave":
        exit()
    elif user_input == "encode":
        encoding_session()
        pass
    elif user_input == "clear":
        os.system('cls' if os.name == 'nt' else 'clear')
        pass
    elif len(user_input) > 0:
        if (len(user_input.split("-"))) < 3:
            print("\n"+mu_symbol+":! Invalid system-code format")
        else:
            name = input(mu_symbol+":! Enter a part name (optional) or press 'Enter' to leave hardware unnamed \n\n    > ")
            if name == "": # If the name input is blank.
                name = "unnamed" # Assigns the to the hardware 'unnamed' as a name.
            hardware = Hardware(name, user_input) # Creates hardware from system-code and assigns some name. # Names are usually but not necessarily unique.
            hardware.mucli_build_hardware(absolute_workspace_path)
    else:
        pass



#!/usr/bin/env python3

import os
from mupy.mucli.selection_branch import SelectionBranch

class CRSPGR022:
    """ This class builds digital representations of CRSPGR022 hardware elements. These representations contain varius objects which help simulate the part."""

    def __init__(self, hardware_code, directory): # Runs when object is initialized.

        self.id = id # This is a unique name or tag referencing a particular object in a set.
        self.hardware_code = hardware_code
        self.family_code = hardware_code.split("-")[0]
        self.type_code = hardware_code.split("-")[1]
        self.errors = []
        self.directory = directory

        self.scad_file_name = directory + "/" + hardware_code +".scad"
        self.scad_file = open(self.scad_file_name, 'w+')  # open file in append mode
        os.system("cp -R "+os.path.dirname(__file__)+"/scad/ "+ self.directory)
        self.scad_file.write('use <scad/CRSPGR022.scad>;\n\n')

        # "CRSPGR022-R-M1-L100H5W25-P15H0"
        if (self.type_code=="R"):
            
            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.length = hardware_code.split("-")[3].split("L")[1].split("H")[0]
            self.height = hardware_code.split("-")[3].split("H")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("W")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("H")[1]
            
            # Replace appropriate 'P' character with decimal point. This is performed after code identification so that to remove inappropriate 'P' characters.
            self.modul = self.modul.replace("P", ".", 1)
            self.length = self.length.replace("P", ".", 1)
            self.height = self.height.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    length = "+self.length+"mm")
            print("    height = "+self.height+"mm")
            print("    width = "+self.width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("")
            
            self.CRSPGR022_R()
            self.scad_file.close()
            
        # "CRSPGR022-MR-M1-L34H34W23-P15H20-P4-PH-F2" 
        # TODO: Getting rendering Errors when automated ; openscad -o xxxx.scad
        elif (self.type_code=="MR"): 

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.length = hardware_code.split("-")[3].split("L")[1].split("H")[0]
            self.height = hardware_code.split("-")[3].split("H")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("W")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("H")[1]
            self.profile = hardware_code.split("-")[5].split("P")[1]
            self.head = hardware_code.split("-")[6]
            self.fastners = hardware_code.split("-")[7].split("F")[1]
            
            # Replace appropriate 'P' character with decimal point. This is performed after code identification so that to remove inappropriate 'P' characters.
            self.modul = self.modul.replace("P", ".", 1)
            self.length = self.length.replace("P", ".", 1)
            self.height = self.height.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.profile = self.profile.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    length = "+self.length+"mm")
            print("    height = "+self.height+"mm")
            print("    width = "+self.width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    profile = "+self.profile)
            print("    head = "+self.head)
            print("    fastners = "+self.fastners)
            print("")
                                    
            self.CRSPGR022_MR()
            self.scad_file.close()

        # "CRSPGR022-HR-M1-L100H5W25-P15H0"
        elif (self.type_code=="HR"): 

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.length = hardware_code.split("-")[3].split("L")[1].split("H")[0]
            self.height = hardware_code.split("-")[3].split("H")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("W")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("H")[1]
            
            # Replace appropriate 'P' character with decimal point. This is performed after code identification so that to remove inappropriate 'P' characters.
            self.modul = self.modul.replace("P", ".", 1)
            self.length = self.length.replace("P", ".", 1)
            self.height = self.height.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    length = "+self.length+"mm")
            print("    height = "+self.height+"mm")
            print("    width = "+self.width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("")
            
            self.CRSPGR022_HR()
            self.scad_file.close()

        # "CRSPGR022-MHR-M1-L34H34W23-P15H20-P4-PH-F2"
        elif (self.type_code=="MHR"):

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.length = hardware_code.split("-")[3].split("L")[1].split("H")[0]
            self.height = hardware_code.split("-")[3].split("H")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("W")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("H")[1]
            self.profile = hardware_code.split("-")[5].split("P")[1]
            self.head = hardware_code.split("-")[6]
            self.fastners = hardware_code.split("-")[7].split("F")[1]
            
            # Replace appropriate 'P' character with decimal point. This is performed after code identification so that to remove inappropriate 'P' characters.
            self.modul = self.modul.replace("P", ".", 1)
            self.length = self.length.replace("P", ".", 1)
            self.height = self.height.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.profile = self.profile.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    length = "+self.length+"mm")
            print("    height = "+self.height+"mm")
            print("    width = "+self.width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    profile = "+self.profile)
            print("    head = "+self.head)
            print("    fastners = "+self.fastners)
            print("")  
            
            self.CRSPGR022_MHR()
            self.scad_file.close()

        # "CRSPGR022-SG-M1-T34W23-P15H20-B4-T"
        elif (self.type_code=="SG"): 

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.tooth_number = hardware_code.split("-")[3].split("T")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("T")[1].split("W")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("P")[1].split("H")[1]
            self.bore = hardware_code.split("-")[5].split("B")[1]
            self.optimized = hardware_code.split("-")[6]
            
            if self.optimized == "T":
                self.optimized = 'true'
            else:
                self.optimized = "false"

            # Replace appropriate 'P' character with decimal point. This is performed after code identification so that to remove inappropriate 'P' characters.
            self.modul = self.modul.replace("P", ".", 1)
            self.tooth_number = self.tooth_number.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.bore = self.bore.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    tooth_number = "+self.tooth_number+" teeth")
            print("    width = "+self.width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    bore diameter= "+self.bore+" mm")
            print("    optimized = "+self.optimized)
            print("")
            
            self.CRSPGR022_SG()
            self.scad_file.close()

        # "CRSPGR022-HG-M1-T34W23-P15H20-B4-T"
        elif (self.type_code=="HG"): 

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.tooth_number = hardware_code.split("-")[3].split("T")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("T")[1].split("W")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("P")[1].split("H")[1]
            self.bore = hardware_code.split("-")[5].split("B")[1]
            self.optimized = hardware_code.split("-")[6]
            
            if self.optimized == "T":
                self.optimized = 'true'
            else:
                self.optimized = "false"
                
            # Replace appropriate 'P' character with decimal point. This is performed after code identification so that to remove inappropriate 'P' characters.
            self.modul = self.modul.replace("P", ".", 1)
            self.tooth_number = self.tooth_number.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.bore = self.bore.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    tooth_number = "+self.tooth_number+" teeth")
            print("    width = "+self.width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    bore diameter= "+self.bore+" mm")
            print("    optimized = "+self.optimized)
            print("")
                
            self.CRSPGR022_HG()
            self.scad_file.close()

        # "CRSPGR022-RG-M1-T34W23R23-P15H20"
        elif (self.type_code=="RG"):

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.tooth_number = hardware_code.split("-")[3].split("T")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("T")[1].split("W")[1].split("R")[0]
            self.rim_width = hardware_code.split("-")[3].split("T")[1].split("W")[1].split("R")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("P")[1].split("H")[1]

            self.modul = self.modul.replace("P", ".", 1)
            self.tooth_number = self.tooth_number.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.rim_width = self.rim_width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    tooth_number = "+self.tooth_number)+" teeth"
            print("    width = "+self.width+"mm")
            print("    rim_width = "+self.rim_width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("")

            self.CRSPGR022_RG()
            self.scad_file.close()

        # "CRSPGR022-HRG-M1-T34W23R23-P15H20"
        elif (self.type_code=="HRG"): 

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.tooth_number = hardware_code.split("-")[3].split("T")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("T")[1].split("W")[1].split("R")[0]
            self.rim_width = hardware_code.split("-")[3].split("T")[1].split("W")[1].split("R")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("P")[1].split("H")[1]
            
            self.modul = self.modul.replace("P", ".", 1)
            self.tooth_number = self.tooth_number.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.rim_width = self.rim_width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    tooth_number = "+self.tooth_number+" teeth")
            print("    width = "+self.width+"mm")
            print("    rim_width = "+self.rim_width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("")

            self.CRSPGR022_HRG()
            self.scad_file.close()

        # "CRSPGR022-PG-M1-S34P34N3W23R23-P15H20-B2-T"
        elif (self.type_code=="PG"): 

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.sun_teeth = hardware_code.split("-")[3].split("S")[1].split("P")[0]
            self.planet_teeth = hardware_code.split("-")[3].split("P")[1].split("N")[0]
            self.number_planets = hardware_code.split("-")[3].split("N")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("W")[1].split("R")[0]
            self.rim_width = hardware_code.split("-")[3].split("R")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("P")[1].split("H")[1]
            self.bore = hardware_code.split("-")[5].split("B")[1]
            self.optimized = hardware_code.split("-")[6]
            
            self.modul = self.modul.replace("P", ".", 1)
            self.optimized = '"'+self.optimized+'"'
            self.width = self.width.replace("P", ".", 1)
            self.rim_width = self.rim_width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.bore = self.bore.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    sun_teeth = "+self.sun_teeth+" sun teeth")
            print("    planet_teeth = "+self.planet_teeth+" planet teeth")
            print("    number_planets = "+self.number_planets+" planets")
            print("    width = "+self.width+"mm")
            print("    rim_width = "+self.rim_width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    bore diameter= "+self.bore+" mm")
            print("")

            self.CRSPGR022_PG()
            self.scad_file.close()

        # "CRSPGR022-BG-M2-T34W25-P25H20C50-B15"
        elif (self.type_code=="BG"):

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.tooth_number = hardware_code.split("-")[3].split("T")[1].split("W")[0]
            self.tooth_width = hardware_code.split("-")[3].split("W")[1].split("S")[0]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("H")[1].split("C")[0]
            self.partial_cone_angle = hardware_code.split("-")[4].split("C")[1]
            self.bore = hardware_code.split("-")[5].split("B")[1]


            self.modul = self.modul.replace("P", ".", 1)
            self.tooth_width = self.tooth_width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.partial_cone_angle = self.partial_cone_angle.replace("P", ".", 1)
            self.bore = self.bore.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    tooth_number = "+self.tooth_number+" teeth")
            print("    tooth_width = "+self.tooth_width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    partial_cone_angle = "+self.partial_cone_angle+"°")
            print("    bore diameter= "+self.bore+" mm")
            print("")

            self.CRSPGR022_BG()
            self.scad_file.close()

        # "CRSPGR022-BHG-M1-T34W50S5P5-P15H20C15-B3"
        elif (self.type_code=="BHG"):

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.tooth_number = hardware_code.split("-")[3].split("T")[1].split("W")[0]
            self.width = hardware_code.split("-")[3].split("W")[1].split("S")[0]
            self.tooth_width = hardware_code.split("-")[3].split("S")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("H")[0]
            self.helix_angle = hardware_code.split("-")[4].split("H")[1].split("C")[0]
            self.partial_cone_angle = hardware_code.split("-")[4].split("C")[1]
            self.bore = hardware_code.split("-")[5].split("B")[1]

            self.modul = self.modul.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.tooth_width = self.tooth_width.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.helix_angle = self.helix_angle.replace("P", ".", 1)
            self.partial_cone_angle = self.partial_cone_angle.replace("P", ".", 1)
            self.bore = self.bore.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    tooth_number = "+self.tooth_number+" teeth")
            print("    width = "+self.width+"mm")
            print("    tooth_width = "+self.tooth_width+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    helix_angle = "+self.helix_angle+"°")
            print("    partial_cone_angle = "+self.partial_cone_angle+"°")
            print("    bore diameter= "+self.bore+" mm")
            print("")

            self.CRSPGR022_BHG()
            self.scad_file.close()

        # "CRSPGR022-W-M1-TH34L23-P15L20-B3"
        elif (self.type_code=="W"):

            self.modul = hardware_code.split("-")[2].split("M")[1]
            self.thread_starts = hardware_code.split("-")[3].split("TH")[1].split("L")[0]
            self.length = hardware_code.split("-")[3].split("TH")[1].split("L")[1]
            self.pressure_angle = hardware_code.split("-")[4].split("P")[1].split("L")[0]
            self.lead_ground = hardware_code.split("-")[4].split("P")[1].split("L")[1]
            self.bore = hardware_code.split("-")[5].split("B")[1]
            
            self.modul = self.modul.replace("P", ".", 1)
            self.thread_starts = self.thread_starts.replace("P", ".", 1)
            self.length = self.length.replace("P", ".", 1)
            self.pressure_angle = self.pressure_angle.replace("P", ".", 1)
            self.lead_ground = self.lead_ground.replace("P", ".", 1)
            self.bore = self.bore.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("    Parameterization Information")
            print("")
            print("    modul = "+self.modul)
            print("    thread_starts = "+self.thread_starts+"mm")
            print("    length = "+self.length+"mm")
            print("    pressure_angle = "+self.pressure_angle+"°")
            print("    lead_ground = "+self.lead_ground)
            print("    bore diameter= "+self.bore+" mm")
            print("")

            self.CRSPGR022_W()
            self.scad_file.close()
            
        else:
            pass

    def CRSPGR022_R(self):

        self.scad_file.write('rack(modul='+str(self.modul)+', length='+str(self.length)+', height='+str(self.height)+', width='+str(self.width)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+');')

    def CRSPGR022_MR(self):

        self.scad_file.write('mountable_rack(modul='+str(self.modul)+', length='+str(self.length)+', height='+str(self.height)+', width='+str(self.width)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+', profile='+str(self.profile)+', head="'+str(self.head)+'",fastners='+str(self.fastners)+');')
    
    def CRSPGR022_HR(self):

        self.scad_file.write('herringbone_rack(modul='+str(self.modul)+', length='+str(self.length)+', height='+str(self.height)+', width='+str(self.width)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+');')

    def CRSPGR022_MHR(self):

        self.scad_file.write('mountable_herringbone_rack(modul='+str(self.modul)+', length='+str(self.length)+', height='+str(self.height)+', width='+str(self.width)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+', profile='+str(self.profile)+', head='+str(self.head)+',fastners='+str(self.fastners)+');')

    def CRSPGR022_SG(self):

        self.scad_file.write('spur_gear(modul='+str(self.modul)+', tooth_number='+str(self.tooth_number)+', width='+str(self.width)+', bore='+str(self.bore)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+', optimized='+str(self.optimized)+');')

    def CRSPGR022_HG(self):

        self.scad_file.write('herringbone_gear(modul='+str(self.modul)+', tooth_number='+str(self.tooth_number)+', width='+str(self.width)+', bore='+str(self.bore)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+', optimized='+str(self.optimized)+');')

    def CRSPGR022_RG(self):

        self.scad_file.write('ring_gear(modul='+str(self.modul)+', tooth_number='+str(self.tooth_number)+', width='+str(self.width)+', rim_width='+str(self.rim_width)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+');')

    def CRSPGR022_HRG(self):

        self.scad_file.write('herringbone_ring_gear(modul='+str(self.modul)+', tooth_number='+str(self.tooth_number)+', width='+str(self.width)+', rim_width='+str(self.rim_width)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+');')

    def CRSPGR022_PG(self):

        self.scad_file.write('planetary_gear(modul='+str(self.modul)+', sun_teeth='+str(self.sun_teeth)+', planet_teeth='+str(self.planet_teeth)+', number_planets='+str(self.number_planets)+', width='+str(self.width)+', rim_width='+str(self.rim_width)+', bore='+str(self.bore)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+', together_built='+'true'+', optimized='+str(self.optimized)+');')

    def CRSPGR022_BG(self):

        self.scad_file.write('bevel_gear(modul='+str(self.modul)+', tooth_number='+str(self.tooth_number)+',  partial_cone_angle='+str(self.partial_cone_angle)+', tooth_width='+str(self.tooth_width)+', bore='+str(self.bore)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+');')

    def CRSPGR022_BHG(self):

        self.scad_file.write('bevel_herringbone_gear(modul='+str(self.modul)+', tooth_number='+str(self.tooth_number)+', partial_cone_angle='+str(self.partial_cone_angle)+', tooth_width='+str(self.tooth_width)+', bore='+str(self.bore)+', pressure_angle='+str(self.pressure_angle)+', helix_angle='+str(self.helix_angle)+');')

    def CRSPGR022_W(self):

        self.scad_file.write('worm(modul='+str(self.modul)+', thread_starts='+str(self.thread_starts)+', length='+str(self.length)+', bore='+str(self.bore)+', pressure_angle='+str(self.pressure_angle)+', lead_angle='+str(self.lead_ground)+');')


class CRSPGR022_encoding:
    def __init__(self):
        self.encoding = ""
    
    def encode_session(self):
        #length = InputBranch("Enter block length in millimeters (mm)")
        #width = InputBranch("Enter width length in millimeters (mm)")
        #height = InputBranch("Enter height length in millimeters (mm)")
        #length.run()
        #width.run()
        #height.run()
        #print("\n")
        #system_code = "CUBX0006-BLK-L"+length.user_input+"W"+width.user_input+"H"+height.user_input
        #self.encoding = system_code
        pass


#!/usr/bin/env python3
import os 
import hashlib
from .coordinates import Coordinates

class Assembly:
    """This class builds json objects which define hardware assembly schemes. 
    These are generally hirearchical. 
    Assemblys are made from aggregations of parts or other assemblys. 
    These positions are typically defined using parametric equations of time $t (referenced in openscad).
    """
    
    def __init__(self, name, system_code = None):
        """Runs at object creation."""

        self.name = name # A name that associates with a part or assembly. it should be human readable and does not need to be unique.
        self.description = "no description" # A description of the object.
        self.coordinate_superset = [] # A list which holds coordinate objects for this assembly.
        self.components = [] # A list which contains the actual part and assembly objects. 
        self.parent = "" # Parent (assembly) ID.
        self.errors = [] # A list that collects errors associated with this object.
        self.directory = "" # Workspace directory.
        self.color = None
        self.system_code = system_code
        self.id = self.assign_id() # Assigns a special
        self.assembly_epoch = 0

    def assign_id(self):
        """ Assigns an appropriate ID with required propertied; 
        1.) maintains uniuness using SHA hashing but is also 
        2.) alphabetic such that no numbers so that scad code can use it for module definitions and 
        3.) Contains a name string so that an aspect of ID is human readable.
        4.) Contains discernable information about weather it is a hardware or assembly object. (In this case the string 'A' or' H' is concated.)
        """
        
        # TODO : Need to create a new hash source with the following properties :
        
        # 1.) If a script is run then if the script doesnt change and is run again the hash must remain the same ; only within the context of a single script regardless of where it is executed creates random like strings when infact the purpose is to allow complexity but still be referenceable post-script run.
        # 2.) Not too sensitive. If some parts are deleted or modified then it would be undesirable for each hash to change. In this way hashes should definatly be dependent on system codes and coordinate sets. Seems strange but these need not be replicated anywhere becasue two objects can not occupy the same space.
        
        hash_source = str(self.system_code)+"-"+str(self.name)
        #hash_source = id(self)
        unrefined_hash = "A"+hashlib.sha256(hash_source.encode('utf-8')).hexdigest() # Combines "A" string to represent that is an assembly, a hash to express uniqueness and a name to express human readability and identification.
        return self.name+"_"+unrefined_hash[:7] # Returns assembly ID.

    def assign_parent_id(self):
        """Empowers each child component a reference to its parent ID ; The ID of this class object."""
        for component in self.components: # For each component in the components of this assembly object.
            component.parent_id = self.id # Parent assembly assigns reference information about itself to it's children.
        pass

    def assign_coordinates_operand_id(self, coordinates):
        """ Assigns coordinate object extra information about the part it operates on. This works by calling a hidden variable in the Coordinates object which expects this function.  """
        coordinates.operand_id = self.id # Assigns an operand ID to the coordinate object.

    def assign_coordinates(self, coordinates): #
        """ Adds another assembly epoche effectivly but it is on the user to utalize the $t time variable properly."""
        self.coordinate_superset.append(coordinates) # Adds a coordinate set to the coordinate superset. TODO : More needs to be done with the coordinate superset.


    def encapsulated_components_scad(self):
        """ Returns scad code representations of parts which are encapsulated."""
        
        encapsulated_components_scad = "" # Declares scad code component.
        for component in self.components: # For each child component in this assembly object.
            encapsulated_components_scad = encapsulated_components_scad + " " + str(component.id)+"();" # scad code which just writes the child scad modules to render and execute within the parent assembly.
        return encapsulated_components_scad # Returns string of scad function calls.


    def assembly_scad(self):
        """ Returns scad code representations of entire assembly."""

        scad_code = '\n/* A module dedicated to this assembly. This module is only called once becasue it references the unique hardware assembly operation (In both real and virtual space) which only needs to be done once. \nThese components are themselves other hardware and assembly objects which carries both python3 and scad representation and eferences other files like these.*/\n'
        scad_code = scad_code + 'module '+str(self.id)+'()\n{\n' # Creates scad codestring to be placed in new file with a file name using the object identification code.


        if (len(self.coordinate_superset)==0): # If there is no coordinate set yet or at all assigned to this hardware object (digital-twin).
            self.assign_coordinates(Coordinates()) # Assign trivial coodinate set; unmoving and existing ($t) being equal from t0 = 0 to tf = 1 .
            

            
        if (len(self.coordinate_superset)==1): # Here you need to add more coordinate sets sepending on choice of epochs.
            
            
            if (self.coordinate_superset[0].t0 > 0): # Here you need to add more coordinate sets sepending on choice of epochs.
                t0 = self.coordinate_superset[0].t0  # Pu0ll from other coordinate information.
                x0 = self.coordinate_superset[0].x0
                y0 = self.coordinate_superset[0].y0
                z0 = self.coordinate_superset[0].z0
                a0 = self.coordinate_superset[0].a0
                b0 = self.coordinate_superset[0].b0
                c0 = self.coordinate_superset[0].c0
                
                self.assign_coordinates(Coordinates(t0 = 0, tf = t0, x0 = x0, y0 =  y0, z0 =  z0, xf = x0, yf =  y0, zf =  z0,  a0 = a0, b0 =  b0, c0 =  c0, af = a0, bf =  b0, cf =  c0))
                
            if (self.coordinate_superset[0].tf < 1): # Here you need to add more coordinate sets sepending on choice of epochs.

                tf = self.coordinate_superset[0].tf  # Pu0ll from other coordinate information.
                tf = self.coordinate_superset[0].tf  # Pu0ll from other coordinate information.
                xf = self.coordinate_superset[0].xf
                yf = self.coordinate_superset[0].yf
                zf = self.coordinate_superset[0].zf
                af = self.coordinate_superset[0].af
                bf = self.coordinate_superset[0].bf
                cf = self.coordinate_superset[0].cf
                
                self.assign_coordinates(Coordinates(t0 = tf, tf = 1, x0 = xf, y0 =  yf, z0 =  zf, xf = xf, yf =  yf, zf =  zf,  a0 = af, b0 =  bf, c0 =  cf, af = af, bf =  bf, cf =  cf))


        for coordinates in self.coordinate_superset: # For each coordinate set.


            scad_code = scad_code + "    /* Animation */\n"

            """This script is contains code which utilizes animation variables ; $t. Using basic linear equations, trajectories are plotted and used for operational and assembly simulations in 3D space."""

            scad_code = scad_code + \
                    '    if ($t >= '+str(coordinates.t0)+' && $t <= '+str(coordinates.tf)+')\n'\
                    '    {\n'
                    
                    
            scad_code = scad_code + "        /* Initialize initial and final position and orientation. These values may be modified for assembly purposes. */\n\n" 

            scad_code = scad_code + "        /* Position */\n"
            scad_code = scad_code + "        x_position_initial = "+str(coordinates.x0)+";    // Initial 'x' position ( in mm ).\n"
            scad_code = scad_code + "        x_position_final = "+str(coordinates.xf)+";      // Final 'x' position ( in mm ).\n"
            scad_code = scad_code + "        y_position_initial = "+str(coordinates.y0)+";    // Initial 'y' position ( in mm ).\n"
            scad_code = scad_code + "        y_position_final = "+str(coordinates.yf)+";      // Final 'y' position ( in mm ).\n"
            scad_code = scad_code + "        z_position_initial = "+str(coordinates.z0)+";    // Initial 'z' position ( in mm ).\n"
            scad_code = scad_code + "        z_position_final = "+str(coordinates.zf)+";      // Final 'z' position ( in mm ).\n\n"

            scad_code = scad_code + "        /* Orientation */\n"
            scad_code = scad_code + "        x_axis_angle_initial = "+str(coordinates.a0)+";  // Initial angle along the 'x' axis. ( in degrees ).\n"
            scad_code = scad_code + "        x_axis_angle_final = "+str(coordinates.af)+";    // Final angle along the 'x' axis. ( in degrees ).\n"
            scad_code = scad_code + "        y_axis_angle_initial = "+str(coordinates.b0)+";  // Initial angle along the 'y' axis. ( in degrees ).\n"
            scad_code = scad_code + "        y_axis_angle_final = "+str(coordinates.bf)+";    // Final angle along the the 'z' axis. ( in degrees ).\n"
            scad_code = scad_code + "        z_axis_angle_initial = "+str(coordinates.c0)+";    // Final angle along the the 'z' axis. ( in degrees ).\n"
            scad_code = scad_code + "        z_axis_angle_final = "+str(coordinates.cf)+";    // Final angle along the 'z' axis. ( in degrees ).\n\n"

            if self.color == None:
                scad_code = scad_code + \
                        '        translate([x_position_initial+($t-'+str(coordinates.t0)+')*(x_position_final-x_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') , y_position_initial+($t-'+str(coordinates.t0)+')*(y_position_final-y_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_position_initial+($t-'+str(coordinates.t0)+')*(z_position_final-z_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') ] ) { rotate([x_axis_angle_initial+($t-'+str(coordinates.t0)+')*(x_axis_angle_final-x_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), y_axis_angle_initial+($t-'+str(coordinates.t0)+')*(y_axis_angle_final-y_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_axis_angle_initial+($t-'+str(coordinates.t0)+')*(z_axis_angle_final-z_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+')]) { '+self.encapsulated_components_scad()+' } }\n'\
                        '    }\n' # scad code.
            else:        
                scad_code = scad_code + \
                        '        translate([x_position_initial+($t-'+str(coordinates.t0)+')*(x_position_final-x_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') , y_position_initial+($t-'+str(coordinates.t0)+')*(y_position_final-y_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_position_initial+($t-'+str(coordinates.t0)+')*(z_position_final-z_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') ] ) { rotate([x_axis_angle_initial+($t-'+str(coordinates.t0)+')*(x_axis_angle_final-x_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), y_axis_angle_initial+($t-'+str(coordinates.t0)+')*(y_axis_angle_final-y_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_axis_angle_initial+($t-'+str(coordinates.t0)+')*(z_axis_angle_final-z_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+')]) { color("'+self.color+'") { '+self.encapsulated_components_scad()+' } } }\n'\
                        '    }\n' # scad code.
                    
        """ This bit here is so the assembly time quantums dont delete the object from ever coming into view."""       
                         
                    
        scad_code = scad_code + '}\n\n' # scad code.
        scad_code = scad_code + '/* Run */\n' # scad code.
        scad_code = scad_code + str(self.id)+'();' # Writes the function calls.
        return scad_code # Returns scad code as a string.

    def assemble(self, directory):
        """ Writes scad file representations of parts and assemblys. This function is called in the generation function in the workspace object."""

        if not os.path.exists(directory): # If directory does not exists.
            os.mkdir(directory) # Create directory for new objects.
    
        # TODO: may need to fix a double slash here.
        scad_file = open(directory+"/"+str(self.id)+".scad", "w") # Creates scad file staging without text.
        scad_file.write("\n/* Manufacturing Utility : Generated hardware assembly .scad file. */\n\n")
        for component in self.components: # For each part.
            scad_file.write('/* Imports assembly module '+component.id+'() from "'+component.id+'.scad" . These components are themselves other hardware and assembly \nobjects which carries both python3 and scad representation and eferences other files like these.*/\n') # A comment describing the import structure and it's relation to the assembly.

            scad_file.write('use <'+component.id+'.scad>;\n') # Import into file.
        scad_file.write(self.assembly_scad()) # Write file.
        scad_file.close() # Close scad file.

    def include(self, component, coordinates): # Includes parts or sub-assemblies with a coordinate set attached to reflect it's position within the assembly.
        """ Includes parts or sub-assemblies with a coordinate set attached to reflect it's position within the assembly."""

        if (type(component).__name__ == "Hardware"): # If it is a part object.
            component.assign_coordinates(coordinates) # Add coordinates to component superset.
            component.parent_assembly_id = self.id
        elif (type(component).__name__ == "Assembly"): # If it is an assembly object.
            component.assign_coordinates(coordinates) # Adds coordinates to component superset.
            component.parent_assembly_id = self.id
        elif (type(component).__name__ == "Coordinates"): # If it is a coordinate set object.
            self.coordinates_superset.append(component.coordinates_json) # Add to coordinate superset.
        else: # Or else..
            pass # Just pass :)
        self.components.append(component) # Add to component set.




#!/usr/bin/env python3
from os.path import dirname, basename, isfile
import glob
modules = glob.glob(dirname(__file__)+"/*.py") # All modules (.py) in this directory (dirname) where '__file__' refers to THIS file.
__all__ = [ basename(f)[:-3] for f in modules if isfile(f) and not f.endswith('__init__.py')]


#!/usr/bin/env python3

from distutils.command.build_scripts import first_line_re
from unicodedata import name
from .decode import Decode
import os

class WorkSpace:
    """  Gives context to parts and assemblys. Provides functionallity by calling sub-functions of assembly and part objects making code much shorter and much more advanced. 
    Provides information such as path information or other metadata. Also serves as a virtual workbench which encapsulates parts and assemblys.
    It is best if workspace is oriented to a certain class of assemblys of projects or just a single assemly or project. It is not nessecary to make a new workspace for 
    every new gearbox unless they are using different parts completely in which case it may only be advisable but certainly not restricted."""

    def __init__(self, directory):
        """When a workspace object is created."""
        self.workspace_directory = directory
        self.hardware = [] # A list of hardware objects contained in this workspace.
        self.assemblies = [] # A list of assembly objects contained in this workspace.
        self.names = []
        self.processed_assemblies = [] # A list of processed assemblies used for recursive import.
        self.errors = [] # Error aggregation object. # TODO : Make a class for this that is more thoughtful.
        self.assembly_epoch_count = 0

    def recursive_import(self, system, system_coordinates):
        """This is a strange function ; it recursivly takes in objects from the main assembly and gives them a home them in the workspace. 
        This function was made to so that users would not need to include every individual part or assembly. """
        first_processed_assemblies_call = True # This is nessary to get things started.  
        while (len(self.processed_assemblies) > 0 or first_processed_assemblies_call == True):

            if (first_processed_assemblies_call==False): # If the first_processed_assemblies_call has been set to false. This is the more likely case but it only happens after the first assembly is identified.
                system = self.processed_assemblies[0] # Set it to the first in the queue.
            else:
                system.assign_coordinates(system_coordinates) # This only happens once which is why this coordinate operation is allowed.

            if (type(system).__name__ == "Hardware"): # If it is a part object.
                print("Workspace : "+self.workspace_directory+" : Discovering hardware object: "+system.id) # Prints status to terminal.
                system.workspace_directory = self.workspace_directory # Assigns workspace directory path to the workspace_directory class variables associated with hardware and assembly component objects.
                self.hardware.append(system) # Append hardware to workspace.
                self.names.append(system.name)
                if (first_processed_assemblies_call==False): # If the first_processed_assemblies_call has been set to false. This is the more likely case but it only happens after the first assembly is identified.
                    self.processed_assemblies.remove(system) # Remove system from processed assemblies list. Remember this was added originally as a component of a larger system.

            elif (type(system).__name__ == "Assembly"): # If it is an assembly object.
                print("Discovering assembly object : "+ system.id) # Prints status to terminal.
                system.workspace_directory = self.workspace_directory # Assigns workspace directory path to the workspace_directory class variables associated with hardware and assembly component objects.
                self.assemblies.append(system) # Append assembly to workspace.
                self.names.append(system.name)
                if (first_processed_assemblies_call==False): # If the first_processed_assemblies_call has been set to false. This is the more likely case but it only happens after the first assembly is identified.
                    self.processed_assemblies.remove(system) # Remove system from processed assemblies list. Remember this was added originally as a component of a larger system.
                
                first_processed_assemblies_call = False # Sets first_processed_assemblies_call flag to False.
                for component in system.components:# Iterates through sub-components of system.
                    self.processed_assemblies.append(component) # Include assembly into the catagories of assemblues which are being processed.
            else: # If there is some other object.
                pass # Just pass.
        

    def detect_duplicates(self, list):    
        ''' Check if given list contains any duplicates '''
        if len(list) == len(set(list)): # If the length of the set is equal to the length of the list.
            return False # There are no duplicate records because the length of the list and set are equal.
        else: # If there is a duplicate.
            self.errors.append("Duplicate hardware or assembly component detected and cannot process assembly script. Remove duplicate component in order to run properly.") # Append error.
            return True # Duplicate record exists.

    def generate(self):
        "Debug function for getting access to system level errors."
        if not (self.detect_duplicates(self.hardware) or self.detect_duplicates(self.assemblies) or self.detect_duplicates(self.names)): # If there are no part duplicates.
            for hardware in self.hardware: # For each part.
                hardware.build_hardware(self.workspace_directory) # Write scad file for this hardwere.
            for assembly in self.assemblies: # For each assembly.
                assembly.assemble(self.workspace_directory) # Write scad file for this assembly.
        else:
            print("Duplicate hardware and/or assemblies detected. Make sure names is globally unique.")

    def print_errors(self):
        "Debug function for getting access to system level erros."
        print(self.errors) # Print erros associated with workspace.
        for hardware in self.hardware: # For each part.
            print(hardware.errors) # Print error.
        for assembly in self.assemblies:  # For each assembly.
            print(assembly.errors) # Print errors contained. (Debug only)

    def run(self, system, coordinates):
        "Main function for workspace to get everthing written and initialized."
        print(system.system_code)
        # TODO: This is where we will need to "call the decode function? (man I had the way I built this interface.)"
        if system.system_code != None:
            # This is where we will need to "call the decode function? (man I had the way I built this interface.)"
            # At this point the workspace does not know anything about the assembly. Only the sysytem code. This is why we need to decode it here before the assemble function is called.
            system_code_decoding = Decode(system.system_code, self.workspace_directory)
            system_code_assembly = system_code_decoding.assembly
            self.recursive_import(system_code_assembly, coordinates) # Recursive inclusion of parts and assemblies into workspace.
            
        else: # In the case that no system code was provided.
            self.recursive_import(system, coordinates) # Recursive inclusion of parts and assemblies into workspace. 
            
        if not os.path.exists(self.workspace_directory): # Checks if main workspace directory exists.
            os.mkdir(self.workspace_directory) # Creates the workspace directory.
        for file in os.listdir(self.workspace_directory):
            if os.path.isfile(os.path.join(self.workspace_directory,file)) == True:
                os.remove(os.path.join(self.workspace_directory, file)) 
                
                   
        self.generate() # Recursive writing of scad object contained in assembly.



#!/usr/bin/env python3
import os # Importsos library. This is useful for running linux commands with python.
from mupy.mucli.input_branch import InputBranch

class CUBX0006:
    """ This class builds digital representations of basic block type elements."""

    def __init__(self, hardware_code, directory):
        """ This class handles decoding of all CUBX0006 family codes. """
        self.id = id # This is a unique name or tag referencing a particular object in a set.
        self.hardware_code = hardware_code # Hardware code fed directly from script or user. 
        
        if len(self.hardware_code.split("-")) < 2: # If there is no type code available.
            print("No typecode")
        else:
            self.type_code = self.hardware_code.split("-")[1] # The second section in the hardware code.
            
        self.directory = directory # Workspace directory to be specific.

        self.scad_file_name = directory +"/"+hardware_code +".scad" # This scad file is used to build the stl. It can be deleted afterwards. # TODO : Delete this file after run() command is called.
        self.scad_file = open(self.scad_file_name, 'w+')  # open file in append mode
        os.system("cp -R "+os.path.dirname(__file__)+"/scad/ "+ self.directory)
        self.scad_file.write('use <scad/CUBX0006.scad>;\n\n') # Write imports.

        if (self.type_code=="BLK"): # Example BLK hardware code : CUBX0006-BLK-L67W796H897
            """Example : CUBX0006-BLK-L67W796H897
               Schema : CUBX0006-BLK-L<Length(mm)>W<Width(mm)>H<Height(mm)>"""

            self.length = hardware_code.split("-")[2].split("L")[1].split("W")[0] # Defines length of a block. Pulled directly from hardware code.
            self.width = hardware_code.split("-")[2].split("W")[1].split("H")[0] # Defines width of a block. Pulled directly from hardware code.
            self.height = hardware_code.split("-")[2].split("H")[1] # Defines height of a box. Pulled directly from hardware code.
            
            
            self.length = self.length.replace("P", ".", 1)
            self.width = self.width.replace("P", ".", 1)
            self.height = self.height.replace("P", ".", 1)

            ''' Testing that code is parsed correctly.'''
            print("    Parameterization Information")
            print("")
            print("    length = "+self.length+"mm")
            print("    width = "+self.width+"mm")
            print("    height = "+self.height+"mm")
            print("")
            
            self.CUBX0006_BLK() # Writes function to scad file with parameters set.

        else:
            pass # Just pass.
    
    def CUBX0006_BLK(self): # Example BLK hardware code : CUBX0006-BLK-L67W796H897
        """  Writes CUBX0006_BLK function to scad file with parameters pulled from hardware code. """
        self.scad_file.write('CUBX0006_BLK(length='+str(self.length).replace('P','.')+', height='+str(self.height).replace('P','.')+', width='+str(self.width).replace('P','.')+');') # Writes function to scad file with parameters set.

class CUBX0006_encoding:
    def __init__(self):
        self.encoding = ""
    
    def encode_session(self):
        length = InputBranch("Enter block length in millimeters (mm)")
        width = InputBranch("Enter width length in millimeters (mm)")
        height = InputBranch("Enter height length in millimeters (mm)")
        length.run()
        width.run()
        height.run()
        system_code = "CUBX0006-BLK-L"+length.user_input+"W"+width.user_input+"H"+height.user_input
        self.encoding = system_code


#!/usr/bin/env python3

""" Required Libray Imports. """
import mupy.core as mu # Imports mupy library.

class CUSBX0177:

    """ This class builds digital representations of basic block type elements."""

    def __init__(self, system_code, workspace_directory):
        
        """ This class handles decoding of all CUBX0006 family codes. """
        self.id = id # This is a unique name or tag referencing a particular object in a set.
        self.system_code = system_code # Hardware code fed directly from script or user. 
        self.family_code = system_code.split("-")[0]
        
        if len(self.system_code.split("-")) < 2: # If there is no type code available.
            print("The provided system-code is invalid because it contained no type-code.")
        else:
            self.type_code = self.system_code.split("-")[1] # The second section in the hardware code.
            
        if len(self.system_code.split("-")) < 3: # If there is no parameterization code available.
            print("The provided system-code is invalid because it contained no parameterization-code.")

            
        self.workspace_directory = workspace_directory # Workspace directory to be specific.

        # CUSBX0177-REG-B25SR2P5-X20Y10Z10
        if (self.type_code=="REG"): # Box panel.

            """ Sets up unit-block length and unit-block shaft-radius. """
            self.unit_block_length = self.system_code.split("-")[2].split("B")[1].split("S")[0]
            self.shaft_radius = self.system_code.split("-")[2].split("SR")[1]
            
            """ Defines the dimensions of the total custom box.in units of unit_bock_length. """
            self.box_x_units = self.system_code.split("-")[3].split("X")[1].split("Y")[0] # Defines the count of 'block units' in the x dimension that make up custom box.
            self.box_y_units = self.system_code.split("-")[3].split("Y")[1].split("Z")[0] # Defines the count of 'block units' in the y dimension that make up custom box.
            self.box_z_units = self.system_code.split("-")[3].split("Z")[1] # Defines the count of 'block units' in the z dimension that make up custom box.

            """ Defines the dimensions of the individual panels making up the custom box which are themselves derived from the box dimensions defined in units of unit_bock_length. """
            panel_a_x_block_units = self.box_x_units # Equation must be satisfied in order to express custom-box function. 
            panel_a_y_block_units = self.box_y_units # Equation must be satisfied in order to express custom-box function. 
            panel_b_x_block_units = self.box_x_units # Equation must be satisfied in order to express custom-box function. 
            panel_b_y_block_units = self.box_z_units # Equation must be satisfied in order to express custom-box function. 
            panel_c_x_block_units = self.box_z_units # Equation must be satisfied in order to express custom-box function. 
            panel_c_y_block_units = self.box_y_units # Equation must be satisfied in order to express custom-box function. 
            panel_d_x_block_units = self.box_x_units # Equation must be satisfied in order to express custom-box function. 
            panel_d_y_block_units = self.box_y_units # Equation must be satisfied in order to express custom-box function. 
            panel_e_x_block_units = self.box_x_units # Equation must be satisfied in order to express custom-box function. 
            panel_e_y_block_units = self.box_z_units # Equation must be satisfied in order to express custom-box function. 
            panel_f_x_block_units = self.box_y_units # Equation must be satisfied in order to express custom-box function. 
            panel_f_y_block_units = self.box_z_units # Equation must be satisfied in order to express custom-box function. 

            """ Define Hardware Components"""
            panel_a = mu.Hardware("panel_a", "CUBX0177-BPAN-B"+str(self.unit_block_length)+"SR"+str(self.shaft_radius)+"-X"+str(panel_a_x_block_units)+"Y"+str(panel_a_y_block_units)+"PP2-RF-SX5Y3-X1Y1-XO0YO0-X30Y30Z20-S") # Constructs panel : This hardware component is a box panel of a CUBX0177 class hardware component. This CUBX0177 component
            panel_b = mu.Hardware("panel_b", "CUBX0177-BPAN-B"+str(self.unit_block_length)+"SR"+str(self.shaft_radius)+"-X"+str(panel_b_x_block_units)+"Y"+str(panel_b_y_block_units)+"PP2-RF-SX5Y3-X1Y1-XO0YO0-X30Y30Z20-S") # Constructs panel A
            panel_c = mu.Hardware("panel_c", "CUBX0177-BPAN-B"+str(self.unit_block_length)+"SR"+str(self.shaft_radius)+"-X"+str(panel_c_x_block_units)+"Y"+str(panel_c_y_block_units)+"PP2-RF-SX5Y10-X1Y1-XO0YO0-X30Y30Z20-S")
            panel_d = mu.Hardware("panel_d", "CUBX0177-BPAN-B"+str(self.unit_block_length)+"SR"+str(self.shaft_radius)+"-X"+str(panel_d_x_block_units)+"Y"+str(panel_d_y_block_units)+"PP2-RF-SX5Y10-X1Y1-XO0YO0-X30Y30Z20-S")
            panel_e = mu.Hardware("panel_e", "CUBX0177-BPAN-B"+str(self.unit_block_length)+"SR"+str(self.shaft_radius)+"-X"+str(panel_e_x_block_units)+"Y"+str(panel_e_y_block_units)+"PP2-RF-SX5Y5-X1Y1-XO0YO0-X30Y30Z20-S")
            panel_f = mu.Hardware("panel_f", "CUBX0177-BPAN-B"+str(self.unit_block_length)+"SR"+str(self.shaft_radius)+"-X"+str(panel_f_x_block_units)+"Y"+str(panel_f_y_block_units)+"PP2-RF-SX5Y5-X1Y1-XO0YO0-X30Y30Z20-S")

            self.unit_block_length = self.unit_block_length.replace("P", ".", 1)
            self.shaft_radius = self.shaft_radius.replace("P", ".", 1)

            """ Define Hardware Components"""
            alpha = 200 # Equation must be satisfied in order to express custom-box function. Alpha is a constant which is useful for describing the distance between panels in the custom box.

            panel_a_assembly_coords = mu.Coordinates(0,1,[0, 0 , float(self.box_z_units) * float(self.unit_block_length) / 2 + alpha],[0, 0 , float(self.box_z_units) * float(self.unit_block_length) / 2],[0,0,0],[0,0,0]) # Equation must be satisfied in order to express custom-box function. 
            panel_b_assembly_coords = mu.Coordinates(0,1,[0, float(self.box_y_units) * float(self.unit_block_length) / 2 + alpha, 0],[0, float(self.box_y_units) * float(self.unit_block_length) / 2, 0],[0,0,0],[-90,0,0]) # Equation must be satisfied in order to express custom-box function. 
            panel_c_assembly_coords = mu.Coordinates(0,1,[ - float(self.box_x_units) * float(self.unit_block_length) / 2 - alpha, 0, 0 ],[ - float(self.box_x_units) * float(self.unit_block_length) / 2, 0, 0 ],[0,0,0],[0,-90,0]) # Equation must be satisfied in order to express custom-box function. 
            panel_d_assembly_coords = mu.Coordinates(0,1,[0 , 0, - float(self.box_z_units) * float(self.unit_block_length) / 2 - alpha],[0 , 0, - float(self.box_z_units) * float(self.unit_block_length) / 2],[0,0,0],[0,180,0]) # Equation must be satisfied in order to express custom-box function. 
            panel_e_assembly_coords = mu.Coordinates(0,1,[ 0, - float(self.box_y_units) * float(self.unit_block_length) / 2 - alpha, 0 ],[ 0, - float(self.box_y_units) * float(self.unit_block_length) / 2, 0 ],[0,0,0],[90,0,0]) # Equation must be satisfied in order to express custom-box function. 
            panel_f_assembly_coords = mu.Coordinates(0,1,[ float(self.box_x_units) * float(self.unit_block_length) / 2 + alpha, 0 ,0 ],[ float(self.box_x_units) * float(self.unit_block_length) / 2, 0 ,0 ],[0,0,0],[90,0,90]) # Equation must be satisfied in order to express custom-box function. 

            """ Define assembly. """
            box_assembly = mu.Assembly("box_assembly")
            box_assembly.include(panel_a, panel_a_assembly_coords)
            box_assembly.include(panel_b, panel_b_assembly_coords)
            box_assembly.include(panel_c, panel_c_assembly_coords)
            box_assembly.include(panel_d, panel_d_assembly_coords)
            box_assembly.include(panel_e, panel_e_assembly_coords)
            box_assembly.include(panel_f, panel_f_assembly_coords)
            
            
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.unit_block_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("")
            
            ''' Execute write scad functions.   '''

            self.assembly = box_assembly



#!/usr/bin/env python3
import os # Importsos library. This is useful for running linux commands with python.
from mupy.mucli.input_branch import InputBranch

class CYLX0006:
    """ This class builds digital representations of basic block type elements."""

    def __init__(self, hardware_code, directory):
        """ This class handles decoding of all CYLX0006 family codes. """
        self.id = id # This is a unique name or tag referencing a particular object in a set.
        self.hardware_code = hardware_code # Hardware code fed directly from script or user. 
        
        if len(self.hardware_code.split("-")) < 2: # If there is no type code available.
            print("There is no type-code present within the provided input.")
        else:
            self.type_code = self.hardware_code.split("-")[1] # The second section in the hardware code.
            
        self.directory = directory # Workspace directory to be specific.

        self.scad_file_name = directory +"/"+hardware_code +".scad" # This scad file is used to build the stl. It can be deleted afterwards. # TODO : Delete this file after run() command is called.
        self.scad_file = open(self.scad_file_name, 'w+')  # open file in append mode
        os.system("cp -R "+os.path.dirname(__file__)+"/scad/ "+ self.directory)
        self.scad_file.write('use <scad/CYLX0006.scad>;\n\n') # Write imports.

        if (self.type_code=="PIP"): # Example PIP hardware code : CYLX0006-PIP-L67W796H897
            
            """Example : CYLX0006-PIP-L44O77I12
               Schema : CYLX0006-PIP-L<Length(mm)>O<OuterRadus(mm)>I<InnerRadus(mm)>"""

            self.length = hardware_code.split("-")[2].split("L")[1].split("O")[0] # Defines length of a block. Pulled directly from hardware code.
            self.outer_radius = hardware_code.split("-")[2].split("O")[1].split("I")[0] # Defines width of a block. Pulled directly from hardware code.
            self.inner_radius = hardware_code.split("-")[2].split("I")[1] # Defines height of a box. Pulled directly from hardware code.
            
            
            self.length = self.length.replace("P", ".", 1)
            self.outer_radius = self.outer_radius.replace("P", ".", 1)
            self.inner_radius = self.inner_radius.replace("P", ".", 1)

            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    length = "+self.length+"mm")
            print("    outer radius = "+self.outer_radius+"mm")
            print("    inner radius = "+self.inner_radius+"mm")
            print("")
            
            self.CYLX0006_PIP() # Writes function to scad file with parameters set.
            
        # CYLX0006_COUP(inner_shaft_radius_a, outer_shaft_radius_a, inner_shaft_depth_a, inner_shaft_radius_b, outer_shaft_radius_b, inner_shaft_depth_b, intershaft_separation_length)
        
        if (self.type_code=="COUP"): # Example PIP hardware code : CYLX0006-PIP-L67W796H897
            
            """Example : CYLX0006-COUP-I25O30D25-I35O40D75-L50
               Schema : CYLX0006-COUP-I<>O<>D<>-I<>O<>D<>-L<>"""

            self.inner_shaft_radius_a = hardware_code.split("-")[2].split("I")[1].split("O")[0] #
            self.outer_shaft_radius_a = hardware_code.split("-")[2].split("O")[1].split("D")[0] #
            self.inner_shaft_depth_a = hardware_code.split("-")[2].split("D")[1] # 
            self.inner_shaft_radius_b = hardware_code.split("-")[3].split("I")[1].split("O")[0] #
            self.outer_shaft_radius_b = hardware_code.split("-")[3].split("O")[1].split("D")[0] #
            self.inner_shaft_depth_b = hardware_code.split("-")[3].split("D")[1] #
            self.intershaft_separation_length = hardware_code.split("-")[4].split("L")[1] #
            
            
            self.inner_shaft_radius_a = self.inner_shaft_radius_a.replace("P", ".", 1)
            self.outer_shaft_radius_a = self.outer_shaft_radius_a.replace("P", ".", 1)
            self.inner_shaft_depth_a = self.inner_shaft_depth_a.replace("P", ".", 1)
            self.inner_shaft_radius_b = self.inner_shaft_radius_b.replace("P", ".", 1)
            self.outer_shaft_radius_b = self.outer_shaft_radius_b.replace("P", ".", 1)
            self.inner_shaft_depth_b = self.inner_shaft_depth_b.replace("P", ".", 1)
            self.intershaft_separation_length = self.intershaft_separation_length.replace("P", ".", 1)
            
            self.total_length = float(self.intershaft_separation_length)+float(self.inner_shaft_depth_a)+float(self.inner_shaft_depth_b)
            
            self.CYLX0006_COUP()

            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    inner-shaft radius A = "+self.inner_shaft_radius_a+"mm")
            print("    outer-shaft radius A = "+self.outer_shaft_radius_a+"mm")
            print("    inner-shaft depth A = "+self.inner_shaft_depth_a+"mm")
            print("    inner-shaft radius B = "+self.inner_shaft_radius_b+"mm")
            print("    outer-shaft radius B = "+self.outer_shaft_radius_b+"mm")
            print("    inner-shaft depth B = "+self.inner_shaft_depth_b+"mm")
            print("    intershaft-separation length = "+self.intershaft_separation_length+"mm")
            print("    total length = "+str(self.total_length)+"mm")
            print("")
        
        else:
            pass # Just pass.
    
    def CYLX0006_PIP(self): # Example PIP hardware code : CYLX0006-PIP-L67W796H897
        """  Writes CYLX0006_PIP function to scad file with parameters pulled from hardware code. """
        self.scad_file.write('CYLX0006_PIP(length='+str(self.length).replace('P','.')+', outer_radius='+str(self.outer_radius).replace('P','.')+', inner_radius='+str(self.inner_radius).replace('P','.')+');') # Writes function to scad file with parameters set.


    def CYLX0006_COUP(self): # Example PIP hardware code : CYLX0006-COUP-I25O30D25-I35O40D75-L50
        """  Writes CYLX0006_PIP function to scad file with parameters pulled from hardware code. """
        self.scad_file.write('CYLX0006_COUP(inner_shaft_radius_a='+str(self.inner_shaft_radius_a).replace('P','.')+', outer_shaft_radius_a='+str(self.outer_shaft_radius_a).replace('P','.')+', inner_shaft_depth_a='+str(self.inner_shaft_depth_a).replace('P','.')+', inner_shaft_radius_b='+str(self.inner_shaft_radius_b).replace('P','.')+', outer_shaft_radius_b='+str(self.outer_shaft_radius_b).replace('P','.')+', inner_shaft_depth_b='+str(self.inner_shaft_depth_b).replace('P','.')+', intershaft_separation_length='+str(self.intershaft_separation_length).replace('P','.')+');') # Writes function to scad file with parameters set.


class CYLX0006_encoding:
    def __init__(self):
        self.encoding = ""
    
    def encode_session(self):
        length = InputBranch("Enter block length in millimeters (mm)")
        width = InputBranch("Enter width length in millimeters (mm)")
        height = InputBranch("Enter height length in millimeters (mm)")
        length.run()
        width.run()
        height.run()
        system_code = "CYLX0006-PIP-L"+length.user_input+"W"+width.user_input+"H"+height.user_input
        self.encoding = system_code


#!/usr/bin/env python3

class Coordinates:
    """ This class builds objects which define a parts or an assembly system coordinate set."""
    
    def __init__(self, t0 = 0, tf = 1, x0 = 0, xf = 0, y0 = 0, yf = 0, z0 = 0, zf = 0, a0 = 0, af = 0, b0 = 0, bf = 0, c0 = 0, cf = 0):
        
        """Temporal (along $t axis/variable) coordinates"""
        
        self.t0 = t0
        self.tf = tf
        
        """Spatial (along x,y,z axes) coordinates"""
        
        self.x0 = x0
        self.y0 = y0
        self.z0 = z0
        self.a0 = a0
        self.b0 = b0
        self.c0 = c0
        
        """Angular (about x,y,z axes) coordinates"""
        
        self.xf = xf
        self.yf = yf
        self.zf = zf
        self.af = af
        self.bf = bf
        self.cf = cf
        
        """Other Information"""

        self.operand_id = "" # The operand_id represents the id of the hardware-unit which the coordinate transformation is operating on.



#!/usr/bin/env python3
import os, sys
import shutil

class BOLTX0004:
    """ This class builds digital representations of BOLTX0004 hardware elements. These representations contain varius objects which help simulate the part."""

    def __init__(self, hardware_code, directory): # Runs when object is initialized.

        self.id = id # This is a unique name or tag referencing a particular object in a set.
        self.hardware_code = hardware_code
        self.family_code = self.hardware_code.split("-")[0]
        self.type_code = self.hardware_code.split("-")[1]
        self.directory = directory
        self.scad_file_name = directory + "/" + hardware_code +".scad" # This scad file is used to build the stl. It can be deleted afterwards. # TODO : Delete this file after run() command is called.
        self.scad_file = open(self.scad_file_name, 'w+')  # open file in append mode

        os.system("cp -R "+os.path.dirname(__file__)+"/scad/ "+ self.directory) # Copies resouces into the workspace directory. These will be deletd later. TODO : Its not a general case that we are in the right directory for this command to work.
        # TODO : Mayneed to put in a double slash test ; replace with one slash incase too many were appended.
        self.scad_file.write('use <scad/BOLTX0004.scad>;\n\n')
        
        
        if (self.type_code=="B1024"): # Example B1024 hardware code : "BOLTX0004-B1024-HEX-L22D10H5"
            
            

            self.bolt_length = self.hardware_code.split("-")[3].split("L")[1].split("D")[0]
            self.head_diameter = self.hardware_code.split("-")[3].split("D")[1].split("H")[0]
            self.head_height =  self.hardware_code.split("-")[3].split("H")[1]
            self.head_shape =  self.hardware_code.split("-")[2]




            """Format Inputs"""

            self.bolt_length = self.bolt_length.replace("P", ".", 1)
            self.head_diameter = self.head_diameter.replace("P", ".", 1)
            self.head_height = self.head_height.replace("P", ".", 1)


            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    family_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    bolt_length = "+self.head_diameter+"mm")
            print("    head_height = "+self.head_height+"mm")
            print("    head_shape = "+self.head_shape)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.BOLTX0004_B1024()
            self.scad_file.close()
            
        if (self.type_code=="B832"): # Example B832 hardware code : "BOLTX0004-B832-HEX-L22D10H5"
            
            

            self.bolt_length = self.hardware_code.split("-")[3].split("L")[1].split("D")[0]
            self.head_diameter = self.hardware_code.split("-")[3].split("D")[1].split("H")[0]
            self.head_height =  self.hardware_code.split("-")[3].split("H")[1]
            self.head_shape =  self.hardware_code.split("-")[2]


            """Format Inputs"""

            self.bolt_length = self.bolt_length.replace("P", ".", 1)
            self.head_diameter = self.head_diameter.replace("P", ".", 1)
            self.head_height = self.head_height.replace("P", ".", 1)


            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    family_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    bolt_length = "+self.head_diameter+"mm")
            print("    head_height = "+self.head_height+"mm")
            print("    head_shape = "+self.head_shape)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.BOLTX0004_B832()
            self.scad_file.close()

        elif (self.type_code=="B632"):
            self.bolt_length = self.hardware_code.split("-")[3].split("L")[1].split("D")[0]
            self.head_diameter = self.hardware_code.split("-")[3].split("D")[1].split("H")[0]
            self.head_height =  self.hardware_code.split("-")[3].split("H")[1]
            self.head_shape =  self.hardware_code.split("-")[2]




            """Format Inputs"""

            self.bolt_length = self.bolt_length.replace("P", ".", 1)
            self.head_diameter = self.head_diameter.replace("P", ".", 1)
            self.head_height = self.head_height.replace("P", ".", 1)


            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    family_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    bolt_length = "+self.head_diameter+"mm")
            print("    head_height = "+self.head_height+"mm")
            print("    head_shape = "+self.head_shape)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.BOLTX0004_B632()
            self.scad_file.close()
            
        if (self.type_code=="B256"):
            self.bolt_length = self.hardware_code.split("-")[3].split("L")[1].split("D")[0]
            self.head_diameter = self.hardware_code.split("-")[3].split("D")[1].split("H")[0]
            self.head_height =  self.hardware_code.split("-")[3].split("H")[1]
            self.head_shape =  self.hardware_code.split("-")[2]




            """Format Inputs"""

            self.bolt_length = self.bolt_length.replace("P", ".", 1)
            self.head_diameter = self.head_diameter.replace("P", ".", 1)
            self.head_height = self.head_height.replace("P", ".", 1)


            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    family_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    bolt_length = "+self.head_diameter+"mm")
            print("    head_height = "+self.head_height+"mm")
            print("    head_shape = "+self.head_shape)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.BOLTX0004_B256()
            self.scad_file.close()
            
        
        if (self.type_code=="N1024"): # Example N1024 hardware code : BOLTX0004-N1024-HEX-D10H5

            self.head_diameter = self.hardware_code.split("-")[3].split("D")[1].split("H")[0]
            self.head_height =  self.hardware_code.split("-")[3].split("H")[1]
            self.head_shape =  self.hardware_code.split("-")[2]




            """Format Inputs"""

            self.head_diameter = self.head_diameter.replace("P", ".", 1)
            self.head_height = self.head_height.replace("P", ".", 1)


            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    family_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    head_height = "+self.head_height+"mm")
            print("    head_diameter = "+self.head_diameter+"mm")
            print("    head_shape = "+self.head_shape)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.BOLTX0004_N1024()
            self.scad_file.close()
            
        if (self.type_code=="N832"): # Flywheel adapter.

            # Example SPAN hardware code : "CUBX0177-FYAD-B25SR2P5".
            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0] # Block length.
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1].replace("P", ".", 1) # Shaft radius.

            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.threshaft_radiusad_starts = self.shaft_radius.replace("P", ".", 1)

            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("")
            
            ''' Execute write scad functions.   '''

            self.BOLTX0004_N832()
            self.scad_file.close()
            
        if (self.type_code=="N632"): # Axle adapter.

            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0] # Block length.
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1].replace("P", ".", 1) # Shaft radius.
            self.padding =  "0."+self.hardware_code.split("-")[3].split("PP")[1]
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''
            
            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.threshaft_radiusad_starts = self.shaft_radius.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("    padding = "+self.padding+"mm")
            print("")
            
            ''' Execute write scad functions.   '''

            self.BOLTX0004_N632()
            self.scad_file.close()


    def BOLTX0004_B1024(self): #
        
        self.scad_file.write('BOLTX0004_B1024( bolt_length = '+self.bolt_length+', head_diameter = '+self.head_diameter+', head_height = '+self.head_height+', head_shape = "'+self.head_shape+'");\n')
   
    def BOLTX0004_B832(self): # 
        self.scad_file.write('BOLTX0004_B832( bolt_length = '+self.bolt_length+', head_diameter = '+self.head_diameter+', head_height = '+self.head_height+', head_shape = "'+self.head_shape+'");\n')

    def BOLTX0004_B632(self):
        self.scad_file.write('BOLTX0004_B632( bolt_length = '+self.bolt_length+', head_diameter = '+self.head_diameter+', head_height = '+self.head_height+', head_shape = "'+self.head_shape+'");\n')
        
    def BOLTX0004_N1024(self):
        self.scad_file.write('BOLTX0004_N1024( head_diameter = '+self.head_diameter+', head_height = '+self.head_height+', head_shape = "'+self.head_shape+'" );\n')

    def BOLTX0004_N832(self):
        self.scad_file.write('BOLTX0004_N832( head_diameter = '+self.head_diameter+', head_height = '+self.head_height+', head_shape = "'+self.head_shape+'" );\n')

    def BOLTX0004_B632(self):
        self.scad_file.write('BOLTX0004_B632( head_diameter = '+self.head_diameter+', head_height = '+self.head_height+', head_shape = "'+self.head_shape+'" );\n')


class BOLTX0004_encoding:
    
    def __init__(self):
        self.type_code
        self.system_code
    
    def encode_session(self):

        #type_code = SelectionBranch("Select CUBX0177 type:")
        #type_code.options.append(["1", "BP", "Box Panel", CUBX0177_BP_input.run])
        #type_code.options.append(["2", "SP", "Simple Panel", exit])
        #type_code.options.append(["3", "SQ", "Square Axle", exit])
        #type_code.options.append(["4", "AA", "Axle Adapter", exit])
        #type_code.options.append(["4", "FA", "Flywheel Adapter", exit])
        #type_code.options.append(["b", "Back", "Navigate to previous menu.", exit])
        
        if (self.type_code=="BPAN"): # Box panel.
            pass
        elif (self.type_code=="SPAN"): # Simple panel.
            pass
        elif (self.type_code=="AXLE"): # Square axle.
            pass
        elif (self.type_code=="FYAD"): # Flywheel adapter.
            pass
        elif (self.type_code=="AXAD"): # Axle adapter.
            pass
        else:
            pass


#!/usr/bin/env python3

from genericpath import isdir
import hashlib
import os, shutil
from .decode import Decode
from os.path import exists
from .coordinates import Coordinates
import urllib.request

class Hardware:
    """ This class builds json objects which define parts. 
        Parts are hardware codes with more identity 
    """
    def __init__(self, name, hardware_code):

        self.hardware_code = hardware_code # A code which references the geometry of the part. Not the manufacturing process or material.
        self.coordinate_superset = [] # The set of coordinate-set objects associated with this hardware object. TODO : This is relativly unstable. 
        self.errors = [] # Error list associated with this object.
        self.name = name # Hardware object name.
        self.id = self.assign_id() # Object ID. Used for identification and uniquness mostly.
        self.material_code = "unspecified" # A code which describes the intended material for this hardware objectto bemade of.
        self.manufacturing_code = "unspecified" # A code which describes the intended manufacturing process for this hardware object. 
        self.workspace_directory = "" # The directory associated with workspace which contains this hardware object.
        self.stl_file_name = self.hardware_code+".stl" # The string path variable associated with the .stl file which this object helps generate.
        self.scad_file_name = self.id+".scad" # The string path variable associated with the scad file which is used to render the .stl file.
        self.color = "white"
        self.stl_imported = False # Determines if the hardware object handles a system-code based design or a custom imported design.
        self.imported_stl = ""
        self.parent_assembly_id = ""

        if hardware_code.endswith('.stl'):
                
            self.stl_imported = True
            self.imported_stl = hardware_code.split("/")[len(hardware_code.split("/"))-1]
            
    def assign_id(self):
        """ Assigns an appropriate ID ; one which maintains uniuness but is also alphabetic ; 
        no numbers so that scad code can use it formodule definitions.
        """
        hash_source = str(self.hardware_code)+"-"+str(self.name)
        #hash_source = id(self)
        unrefined_hash = "P"+hashlib.sha256(hash_source.encode('utf-8')).hexdigest() #
        return self.name+"_"+unrefined_hash[:7] #
    
    def assign_coordinates(self, coordinates): #
        """ Adds another assembly epoche effectivly but it is on the user to utilize the $t time variable properly."""
        self.coordinate_superset.append(coordinates) #

    def assign_coordinates_operand_id(self, coordinates):
        coordinates.operand_id = self.id
        """ Assigns coordinate object extra information about the part it operates on. This works by calling a hidden variable in the Coordinates object which expects this function.  """

    def hardware_scad(self): # Retuns scad code and imports the hardware code specified.
        """ Retuns scad code and imports the hardware code specified."""
        scad_code = '\n/* A module dedicated to this hardware object. This module is only called once becasue it references the unique hardware element (In both real and virtual space) which only needs to be done once. */\n\n'
        scad_code = 'module '+self.id+'()\n{\n' #
        if (len(self.coordinate_superset)==0):
            self.assign_coordinates(Coordinates())     
            
        if (len(self.coordinate_superset)==1): # Here you need to add more coordinate sets sepending on choice of epochs.
            
            if (self.coordinate_superset[0].t0 > 0): # Here you need to add more coordinate sets sepending on choice of epochs.
                t0 = self.coordinate_superset[0].t0  # Pu0ll from other coordinate information.
                x0 = self.coordinate_superset[0].x0
                y0 = self.coordinate_superset[0].y0
                z0 = self.coordinate_superset[0].z0
                a0 = self.coordinate_superset[0].a0
                b0 = self.coordinate_superset[0].b0
                c0 = self.coordinate_superset[0].c0
                
                self.assign_coordinates(Coordinates(t0 = 0, tf = t0, x0 = x0, y0 =  y0, z0 =  z0, xf = x0, yf =  y0, zf =  z0,  a0 = a0, b0 =  b0, c0 =  c0, af = a0, bf =  b0, cf =  c0))
                
            if (self.coordinate_superset[0].tf < 1): # Here you need to add more coordinate sets sepending on choice of epochs.

                tf = self.coordinate_superset[0].tf  # Pu0ll from other coordinate information.
                tf = self.coordinate_superset[0].tf  # Pu0ll from other coordinate information.
                xf = self.coordinate_superset[0].xf
                yf = self.coordinate_superset[0].yf
                zf = self.coordinate_superset[0].zf
                af = self.coordinate_superset[0].af
                bf = self.coordinate_superset[0].bf
                cf = self.coordinate_superset[0].cf
                
                self.assign_coordinates(Coordinates(t0 = tf, tf = 1, x0 = xf, y0 =  yf, z0 =  zf, xf = xf, yf =  yf, zf =  zf,  a0 = af, b0 =  bf, c0 =  cf, af = af, bf =  bf, cf =  cf))

        for coordinates in self.coordinate_superset: #

            scad_code = scad_code + "    /* Animation Sequence*/\n"
            scad_code = scad_code + \
                    '  if ($t >= '+str(coordinates.t0)+' && $t <= '+str(coordinates.tf)+')\n'\
                    '  {\n'      
            scad_code = scad_code + "    /* Initialize initial and final position and orientation. These values may be modified for assembly purposes. */\n\n" 
    
            scad_code = scad_code + "        /* Position */\n"
            scad_code = scad_code + "        x_position_initial = "+str(coordinates.x0)+";    // Initial 'x' position ( in mm ).\n"
            scad_code = scad_code + "        x_position_final = "+str(coordinates.xf)+";      // Final 'x' position ( in mm ).\n"
            scad_code = scad_code + "        y_position_initial = "+str(coordinates.y0)+";    // Initial 'y' position ( in mm ).\n"
            scad_code = scad_code + "        y_position_final = "+str(coordinates.yf)+";      // Final 'y' position ( in mm ).\n"
            scad_code = scad_code + "        z_position_initial = "+str(coordinates.z0)+";    // Initial 'z' position ( in mm ).\n"
            scad_code = scad_code + "        z_position_final = "+str(coordinates.zf)+";      // Final 'z' position ( in mm ).\n\n"

            scad_code = scad_code + "        /* Orientation */\n"
            scad_code = scad_code + "        x_axis_angle_initial = "+str(coordinates.a0)+";  // Initial angle along the 'x' axis. ( in degrees ).\n"
            scad_code = scad_code + "        x_axis_angle_final = "+str(coordinates.af)+";    // Final angle along the 'x' axis. ( in degrees ).\n"
            scad_code = scad_code + "        y_axis_angle_initial = "+str(coordinates.b0)+";  // Initial angle along the 'y' axis. ( in degrees ).\n"
            scad_code = scad_code + "        y_axis_angle_final = "+str(coordinates.bf)+";    // Final angle along the the 'z' axis. ( in degrees ).\n"
            scad_code = scad_code + "        z_axis_angle_initial = "+str(coordinates.c0)+";    // Final angle along the the 'z' axis. ( in degrees ).\n"
            scad_code = scad_code + "        z_axis_angle_final = "+str(coordinates.cf)+";    // Final angle along the 'z' axis. ( in degrees ).\n\n"
        
            if self.stl_imported == False:
                scad_code = scad_code + '    translate([x_position_initial+($t-'+str(coordinates.t0)+')*(x_position_final-x_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') , y_position_initial+($t-'+str(coordinates.t0)+')*(y_position_final-y_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_position_initial + ($t-'+str(coordinates.t0)+')*(z_position_final-z_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') ] ) { rotate([x_axis_angle_initial+($t-'+str(coordinates.t0)+')*(x_axis_angle_final-x_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), y_axis_angle_initial+($t-'+str(coordinates.t0)+')*(y_axis_angle_final-y_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_axis_angle_initial+($t-'+str(coordinates.t0)+')*(z_axis_angle_final-z_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+')])'+' { color("'+self.color+'") { import("stl_files/'+ self.hardware_code + '.stl"); } } }\n''  }\n' # translation block scad code.
            elif self.stl_imported == True: 
                scad_code = scad_code + '    translate([x_position_initial+($t-'+str(coordinates.t0)+')*(x_position_final-x_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') , y_position_initial+($t-'+str(coordinates.t0)+')*(y_position_final-y_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_position_initial + ($t-'+str(coordinates.t0)+')*(z_position_final-z_position_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+') ] ) { rotate([x_axis_angle_initial+($t-'+str(coordinates.t0)+')*(x_axis_angle_final-x_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), y_axis_angle_initial+($t-'+str(coordinates.t0)+')*(y_axis_angle_final-y_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+'), z_axis_angle_initial+($t-'+str(coordinates.t0)+')*(z_axis_angle_final-z_axis_angle_initial)/('+str(coordinates.tf)+'-'+str(coordinates.t0)+')])'+' { color("'+self.color+'") { import("stl_files/'+ self.imported_stl + '"); } } }\n''  }\n' # translation block scad code. 
                    
        """ This bit here is so the assembly time quantums dont delete the object from ever coming into view."""            
                
        scad_code = scad_code + '}\n' # Finishes themodule closing bracket.
        scad_code = scad_code + '/* Run */\n' # scad code.
        scad_code = scad_code + self.id+'();' # Writes the scad code used to call the function.

        return scad_code # Returns scad code.
    
    def write_scad(self, directory):
        """Writes .scad code for this hardware object."""
        
        if not os.path.exists(directory): # Checks if main workspace directory exists.
            os.mkdir(directory) # Creates the workspace directory.

        if not os.path.exists(directory+"/stl_files/"): # Checks if the stl_file folder exists.
            os.mkdir(directory+"/stl_files/") # Creates the stl_files directory.

        scad_file = open(directory+"/"+self.scad_file_name, "w") # Creates new .scad file.
        scad_file.write("\n/* Parametric System Instructions : Generated hardware .scad file. */\n\n")
        scad_file.write(self.hardware_scad()) # Writes scad code to file.
        scad_file.close() # Close file.
    
    def build_stl(self, directory):
        """"""
        mu_symbol = '\u00B5'
        #TODO: Make a downloading statement for downloaded or imported.
        print("Decoding : "+self.hardware_code)
        decoding = Decode(self.hardware_code, directory) # This is where the hardware code is converted into an scad code which can be rendered to an stl.
        if not exists(directory+"/stl_files/"+self.hardware_code+".stl") and decoding.family_code_valid == True: #
            print("Rendering : "+self.hardware_code)
            print(""+mu_symbol+":! openscad -o "+directory+"/stl_files/"+self.hardware_code+".stl "+directory+"/"+self.hardware_code+".scad") # This is the scad file used to create the stl files
            os.system("openscad -o "+directory+"/stl_files/"+self.hardware_code+".stl "+directory+"/"+self.hardware_code+".scad") # This is the scad file used to create the stl files
                    
        if self.stl_imported == True:
            

            if len(self.hardware_code.split("http")) > 0:
                stl_file = self.hardware_code
                urllib.request.urlretrieve(stl_file, directory+"/stl_files/"+self.imported_stl)
            else:
                shutil.copy2(self.imported_stl, directory+"/stl_files/")            

            
    def mucli_build_stl(self, directory):
        """"""
        mu_symbol = '\u00B5'
        decoding = Decode(self.hardware_code, directory) # This is where the hardware code is converted into an scad code which can be rendered to an stl.
        if not exists(directory+"/stl_files/"+self.hardware_code+".stl") and decoding.family_code_valid == True: #
        
            print("")
            print(""+mu_symbol+":! Rendering : "+self.hardware_code)
            print(""+mu_symbol+":! openscad -o "+directory+"/stl_files/"+self.hardware_code+".stl "+directory+"/"+self.hardware_code+".scad") # This is the scad file used to create the stl files
            os.system("openscad -o "+directory+"/stl_files/"+self.hardware_code+".stl "+directory+"/"+self.hardware_code+".scad") # This is the scad file used to create the stl files
            print("")
            
        if decoding.family_code_valid == False:
            print("\n"+mu_symbol+":! Error : The entered system code contained an unrecognized family code\n")
    
    def remove_jig(self, directory): # Called with each run command per part.
        """Removes files which have served thier purpose and are no longer needed."""
        if exists(directory+self.hardware_code+".scad"):# Checks for the existence of an .scad file in the workspace with name equal to that of the hardware code.
            try: # Attempt to delete scad file jig.
                os.remove(directory+self.hardware_code+".scad") # Remove scad file with a name equal to that of the hardware code. This file is part of the jig.
            except OSError: # If there is an error with attempting to delete the scad needed to render the STL.
                print(Exception) # Prints exception.
        if exists(directory+"scad"): # Checks if scad directory is present in the workspace directory.
            shutil.rmtree(directory+"scad") # Remove scad directory. TODO : Check this the path may of changed since the reorg.

    def build_hardware(self, directory):
        """Clean function which builds and manages stl files """
        self.write_scad(directory) # Write scad files nessecarry to build stl. These files are temporary and the name of these files are equivilent to the hardware code currently.
        self.build_stl(directory) # Builds stl from hardware code.
        self.remove_jig(directory) # Removes unnessecary files which were used to buld the stl.
        
    def mucli_build_hardware(self, directory):
        """Clean function which builds and manages stl files """
        self.write_scad(directory) # Write scad files nessecarry to build stl. These files are temporary and the name of these files are equivilent to the hardware code currently.
        self.mucli_build_stl(directory) # Builds stl from hardware code.
        self.remove_jig(directory) # Removes unnessecary files which were used to buld the stl.
 


#!/usr/bin/env python3
class InputBranch:
    """Creates objects which inject themselves into some shell or terminal type enviornment and asks users for input."""
 
    def __init__(self, question): # Run at object creation.
        """Runs when class object is created."""
        self.question = question # A question that prompts the user for some meaningful input.
        self.user_input = "" # A string given by the user corresponding to self.question when run() is called.
        
    def run(self): # Run user input class function.
        """Prompts user for some alphanumeric input specified by self.question."""
        print("  "+self.question+"\n") # Prompts user.
        self.user_input = input("  > ") # Takes user input.
        print("") # Provides  newline.




#!/usr/bin/env python3
import os, sys
import shutil

class CUBX0177:
    """ This class builds digital representations of CUBX0177 hardware elements. These representations contain varius objects which help simulate the part."""

    def __init__(self, hardware_code, directory): # Runs when object is initialized.

        self.id = id # This is a unique name or tag referencing a particular object in a set.
        self.hardware_code = hardware_code
        self.family_code = self.hardware_code.split("-")[0]
        self.type_code = self.hardware_code.split("-")[1]
        self.directory = directory
        self.scad_file_name = directory + "/" + hardware_code +".scad" # This scad file is used to build the stl. It can be deleted afterwards. # TODO : Delete this file after run() command is called.
        self.scad_file = open(self.scad_file_name, 'w+')  # open file in append mode

        os.system("cp -R "+os.path.dirname(__file__)+"/scad/ "+ self.directory) # Copies resouces into the workspace directory. These will be deletd later. TODO : Its not a general case that we are in the right directory for this command to work.
        # TODO : Mayneed to put in a double slash test ; replace with one slash incase too many were appended.
        self.scad_file.write('use <scad/CUBX0177.scad>;\n\n')
        if (self.type_code=="BPAN"): # Box panel.
            
            # Example BPAN hardware code : "CUBX0177-BPAN-B25SR2P5-X8Y8PP2-RT-SX25Y25-X8Y8-XO3YO5-X20Y20Z10-S"

            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0]
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1]
            self.x_units =  self.hardware_code.split("-")[3].split("X")[1].split("Y")[0]
            self.y_units =  self.hardware_code.split("-")[3].split("Y")[1].split("P")[0]
            self.padding =  "0."+self.hardware_code.split("-")[3].split("PP")[1]

            if (self.hardware_code.split("-")[4].count("R") == 1):
                self.orientation = '"regular"' # Regular orientation.
            elif (self.hardware_code.split("-")[4].count("I") == 1):
                self.orientation = '"inverted"' # Inverted orientation.
            elif (self.hardware_code.split("-")[4].count("C") == 1):
                self.orientation = '"chimera"' # Chimera orientation.
            elif (self.hardware_code.split("-")[4].count("X") == 1):
                self.orientation = '"inverted_chimera"' # Inverted chimera.
            else:
                self.errors.append("Incorrect orientation code.")
                pass
            if (self.hardware_code.split("-")[4].count("T") == 1):
                self.teeth = "true"
            elif (self.hardware_code.split("-")[4].count("F") == 1):
                self.teeth = "false"
            else:
                self.errors.append("Incorrect teeth code.")
                pass
            
            """Draw Inputs From System-Code"""

            self.x_cavity_spacing =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[0]
            self.y_cavity_spacing =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[1]
            self.x_cavity_units =  self.hardware_code.split("-")[6].split("X")[1].split("Y")[0]
            self.y_cavity_units =  self.hardware_code.split("-")[6].split("X")[1].split("Y")[1]
            self.x_offset  =  self.hardware_code.split("-")[7].split("XO")[1].split("YO")[0]
            self.y_offset  =  self.hardware_code.split("-")[7].split("YO")[1]
            self.x_cavity_dimensions =  self.hardware_code.split("-")[8].split("X")[1].split("Y")[0]
            self.y_cavity_dimensions =  self.hardware_code.split("-")[8].split("X")[1].split("Y")[1].split("Z")[0]
            self.z_cavity_dimensions =  self.hardware_code.split("-")[8].split("X")[1].split("Y")[1].split("Z")[1]

            if (self.hardware_code.split("-")[9].count("S") == 1):
                self.cavity_type = '"S"'
            elif (self.hardware_code.split("-")[9].count("C") == 1):
                self.cavity_type = '"C"'
            else:
                self.errors.append("Incorrect cavity code.")
                pass


            """Format Inputs"""

            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.shaft_radius = self.shaft_radius.replace("P", ".", 1)
            self.x_cavity_spacing = self.x_cavity_spacing.replace("P", ".", 1)
            self.y_cavity_spacing = self.y_cavity_spacing.replace("P", ".", 1)
            self.x_offset = self.x_offset.replace("P", ".", 1)
            self.y_offset = self.y_offset.replace("P", ".", 1)
            self.x_cavity_dimensions = self.x_cavity_dimensions.replace("P", ".", 1)
            self.y_cavity_dimensions = self.y_cavity_dimensions.replace("P", ".", 1)
            self.z_cavity_dimensions = self.z_cavity_dimensions.replace("P", ".", 1)




            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("    x_units = "+self.x_units)
            print("    y_units = "+self.y_units)
            print("    padding = "+self.padding+"mm")
            print("    orientation = "+self.orientation)
            print("    teeth = "+self.teeth)
            print("    x_cavity_spacing = "+self.x_cavity_spacing+"mm")
            print("    y_cavity_spacing = "+self.y_cavity_spacing+"mm")
            print("    x_cavity_units = "+self.x_cavity_units)
            print("    y_cavity_units = "+self.y_cavity_units)
            print("    x_cavity_dimensions = "+self.x_cavity_dimensions+"mm")
            print("    y_cavity_dimensions = "+self.y_cavity_dimensions+"mm")
            print("    z_cavity_dimensions = "+self.z_cavity_dimensions+"mm")
            print("    cavity_type = "+self.cavity_type)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.CUBX0177_BPAN()
            self.scad_file.close()
            
        if (self.type_code=="BPANS2"): # Box panel.
            
            # Example BPANS2 hardware code : "CUBX0177-BPANS2-B7P5SR2P2-X8Y8PP21-IF-SX0Y0-X1Y1-XO0YO0-X19Y19Z1P5-C-SX0Y0-X1Y1-XO0YO0-X10Y10Z1P10-C"

            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0]
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1]
            self.x_units =  self.hardware_code.split("-")[3].split("X")[1].split("Y")[0]
            self.y_units =  self.hardware_code.split("-")[3].split("Y")[1].split("P")[0]
            self.padding =  "0."+self.hardware_code.split("-")[3].split("PP")[1]

            if (self.hardware_code.split("-")[4].count("R") == 1):
                self.orientation = '"regular"' # Regular orientation.
            elif (self.hardware_code.split("-")[4].count("I") == 1):
                self.orientation = '"inverted"' # Inverted orientation.
            elif (self.hardware_code.split("-")[4].count("C") == 1):
                self.orientation = '"chimera"' # Chimera orientation.
            elif (self.hardware_code.split("-")[4].count("X") == 1):
                self.orientation = '"inverted_chimera"' # Inverted chimera.
            else:
                self.errors.append("Incorrect orientation code.")
                pass
            if (self.hardware_code.split("-")[4].count("T") == 1):
                self.teeth = "true"
            elif (self.hardware_code.split("-")[4].count("F") == 1):
                self.teeth = "false"
            else:
                self.errors.append("Incorrect teeth code.")
                pass
            
            """Draw Inputs From System-Code"""

            self.x_cavity_spacing_1 =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[0]
            self.y_cavity_spacing_1 =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[1]
            self.x_cavity_units_1 =  self.hardware_code.split("-")[6].split("X")[1].split("Y")[0]
            self.y_cavity_units_1 =  self.hardware_code.split("-")[6].split("X")[1].split("Y")[1]
            self.x_offset_1  =  self.hardware_code.split("-")[7].split("XO")[1].split("YO")[0]
            self.y_offset_1  =  self.hardware_code.split("-")[7].split("YO")[1]
            self.x_cavity_dimensions_1 =  self.hardware_code.split("-")[8].split("X")[1].split("Y")[0]
            self.y_cavity_dimensions_1 =  self.hardware_code.split("-")[8].split("X")[1].split("Y")[1].split("Z")[0]
            self.z_cavity_dimensions_1 =  self.hardware_code.split("-")[8].split("X")[1].split("Y")[1].split("Z")[1]

            if (self.hardware_code.split("-")[9].count("S") == 1):
                self.cavity_type_1 = '"S"'
            elif (self.hardware_code.split("-")[9].count("C") == 1):
                self.cavity_type_1 = '"C"'
            else:
                self.errors.append("Incorrect cavity code 1.")
                pass
            
            
            self.x_cavity_spacing_2 =  self.hardware_code.split("-")[10].split("X")[1].split("Y")[0]
            self.y_cavity_spacing_2 =  self.hardware_code.split("-")[10].split("X")[1].split("Y")[1]
            self.x_cavity_units_2 =  self.hardware_code.split("-")[11].split("X")[1].split("Y")[0]
            self.y_cavity_units_2 =  self.hardware_code.split("-")[11].split("X")[1].split("Y")[1]
            self.x_offset_2  =  self.hardware_code.split("-")[12].split("XO")[1].split("YO")[0]
            self.y_offset_2  =  self.hardware_code.split("-")[12].split("YO")[1]
            self.x_cavity_dimensions_2 =  self.hardware_code.split("-")[13].split("X")[1].split("Y")[0]
            self.y_cavity_dimensions_2 =  self.hardware_code.split("-")[13].split("X")[1].split("Y")[1].split("Z")[0]
            self.z_cavity_dimensions_2 =  self.hardware_code.split("-")[13].split("X")[1].split("Y")[1].split("Z")[1]

            if (self.hardware_code.split("-")[14].count("S") == 1):
                self.cavity_type_2 = '"S"'
            elif (self.hardware_code.split("-")[14].count("C") == 1):
                self.cavity_type_2 = '"C"'
            else:
                self.errors.append("Incorrect cavity code 2.")
                pass


            """Format Inputs"""
            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.shaft_radius = self.shaft_radius.replace("P", ".", 1)
            self.x_cavity_spacing_1 = self.x_cavity_spacing_1.replace("P", ".", 1)
            self.y_cavity_spacing_1 = self.y_cavity_spacing_1.replace("P", ".", 1)
            self.x_offset_1 = self.x_offset_1.replace("P", ".", 1)
            self.y_offset_1 = self.y_offset_1.replace("P", ".", 1)
            self.x_cavity_dimensions_1 = self.x_cavity_dimensions_1.replace("P", ".", 1)
            self.y_cavity_dimensions_1 = self.y_cavity_dimensions_1.replace("P", ".", 1)
            self.z_cavity_dimensions_1 = self.z_cavity_dimensions_1.replace("P", ".", 1)

            self.x_cavity_spacing_2 = self.x_cavity_spacing_2.replace("P", ".", 1)
            self.y_cavity_spacing_2 = self.y_cavity_spacing_2.replace("P", ".", 1)
            self.x_offset_2 = self.x_offset_2.replace("P", ".", 1)
            self.y_offset_2 = self.y_offset_2.replace("P", ".", 1)
            self.x_cavity_dimensions_2 = self.x_cavity_dimensions_2.replace("P", ".", 1)
            self.y_cavity_dimensions_2 = self.y_cavity_dimensions_2.replace("P", ".", 1)
            self.z_cavity_dimensions_2 = self.z_cavity_dimensions_2.replace("P", ".", 1)




            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("    x_units = "+self.x_units)
            print("    y_units = "+self.y_units)
            print("    padding = "+self.padding+"mm")
            print("    orientation = "+self.orientation)
            print("    teeth = "+self.teeth)
            print("    x_cavity_spacing_1 = "+self.x_cavity_spacing_1+"mm")
            print("    y_cavity_spacing_1 = "+self.y_cavity_spacing_1+"mm")
            print("    x_cavity_units_1 = "+self.x_cavity_units_1)
            print("    y_cavity_units_1 = "+self.y_cavity_units_1)
            print("    x_cavity_dimensions_1 = "+self.x_cavity_dimensions_1+"mm")
            print("    y_cavity_dimensions_1 = "+self.y_cavity_dimensions_1+"mm")
            print("    z_cavity_dimensions_1 = "+self.z_cavity_dimensions_1+"mm")
            print("    cavity_type_1 = "+self.cavity_type_1)
            print("    x_cavity_spacing_2 = "+self.x_cavity_spacing_2+"mm")
            print("    y_cavity_spacing_2 = "+self.y_cavity_spacing_2+"mm")
            print("    x_cavity_units_2 = "+self.x_cavity_units_2)
            print("    y_cavity_units_2 = "+self.y_cavity_units_2)
            print("    x_cavity_dimensions_2 = "+self.x_cavity_dimensions_2+"mm")
            print("    y_cavity_dimensions_2 = "+self.y_cavity_dimensions_2+"mm")
            print("    z_cavity_dimensions_2 = "+self.z_cavity_dimensions_2+"mm")
            print("    cavity_type_2 = "+self.cavity_type_2)
            print("")
            
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.CUBX0177_BPANS2()
            self.scad_file.close()

        elif (self.type_code=="SPAN"): # Simple panel.
            # CUBX0177_SPAN(block_length, shaft_radius, x_units, y_units, x_spacing, y_spacing, x_cavity_units, y_cavity_units, x_offset, y_offset, x_cavity_dimensions, y_cavity_dimensions, z_cavity_dimensions, cavity_type)
            # Example SPAN hardware code : "CUBX0177-SPAN-B7P5SR2P18-X9Y9-SX15Y15-X4Y4-XO0YO0-X7P6Y7P6Z10-S"

            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0]
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1]
            self.x_units =  self.hardware_code.split("-")[3].split("X")[1].split("Y")[0]
            self.y_units =  self.hardware_code.split("-")[3].split("Y")[1].split("P")[0]

            self.x_cavity_spacing =  self.hardware_code.split("-")[4].split("X")[1].split("Y")[0]
            self.y_cavity_spacing =  self.hardware_code.split("-")[4].split("X")[1].split("Y")[1]
            self.x_cavity_units =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[0]
            self.y_cavity_units =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[1]
            self.x_offset  =  self.hardware_code.split("-")[6].split("XO")[1].split("YO")[0]
            self.y_offset  =  self.hardware_code.split("-")[6].split("YO")[1]
            self.x_cavity_dimensions =  self.hardware_code.split("-")[7].split("X")[1].split("Y")[0]
            self.y_cavity_dimensions =  self.hardware_code.split("-")[7].split("X")[1].split("Y")[1].split("Z")[0]
            self.z_cavity_dimensions =  self.hardware_code.split("-")[7].split("X")[1].split("Y")[1].split("Z")[1]

            if (self.hardware_code.split("-")[8].count("S") == 1):
                self.cavity_type = '"S"'
            elif (self.hardware_code.split("-")[8].count("C") == 1):
                self.cavity_type = '"C"'
            else:
                self.errors.append("Incorrect cavity code.")
                pass

            """Format Inputs"""

            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.shaft_radius = self.shaft_radius.replace("P", ".", 1)
            self.x_cavity_spacing = self.x_cavity_spacing.replace("P", ".", 1)
            self.y_cavity_spacing = self.y_cavity_spacing.replace("P", ".", 1)
            self.x_offset = self.x_offset.replace("P", ".", 1)
            self.y_offset = self.y_offset.replace("P", ".", 1)
            self.x_cavity_dimensions = self.x_cavity_dimensions.replace("P", ".", 1)
            self.y_cavity_dimensions = self.y_cavity_dimensions.replace("P", ".", 1)
            self.z_cavity_dimensions = self.z_cavity_dimensions.replace("P", ".", 1)

            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.CUBX0177_SPAN()
            self.scad_file.close()
            
        if (self.type_code=="SPANS2"): # Simple panel.
            # Example SPANS2 hardware code : "CUBX0177-SPANS2-B7P5SR2P2-X8Y8-SX0Y0-X1Y1-XO0YO0-X19Y19Z1P5-C-SX0Y0-X1Y1-XO0YO0-X10Y10Z1P10-C"
            # CUBX0177_SPANS2( block_length = '+self.block_unit_length+', shaft_radius = '+self.shaft_radius+', x_units = '+self.x_units+', y_units = '+self.y_units+', x_cavity_spacing_1 = '+self.x_cavity_spacing_1+', y_cavity_spacing_1 = '+self.y_cavity_spacing_1+', x_cavity_units_1 = '+self.x_cavity_units_1+', y_cavity_units_1 = '+self.y_cavity_units_1+', x_offset_1 = '+self.x_offset_1+', y_offset_1 = '+self.y_offset_1+', x_cavity_dimensions_1 = '+self.x_cavity_dimensions_1+', y_cavity_dimensions_1 = '+self.y_cavity_dimensions_1+', z_cavity_dimensions_1 = '+self.z_cavity_dimensions_1+', cavity_type_1 = '+self.cavity_type_1+', x_cavity_spacing_2 = '+self.x_cavity_spacing_2+', y_cavity_spacing_2 = '+self.y_cavity_spacing_2+', x_cavity_units_2 = '+self.x_cavity_units_2+', y_cavity_units_2 = '+self.y_cavity_units_2+', x_offset_2 = '+self.x_offset_2+', y_offset_2 = '+self.y_offset_2+', x_cavity_dimensions_2 = '+self.x_cavity_dimensions_2+', y_cavity_dimensions_2 = '+self.y_cavity_dimensions_2+', z_cavity_dimensions_2 = '+self.z_cavity_dimensions_2+', cavity_type_2 = '+self.cavity_type_2+' );\n')
            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0]
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1]
            self.x_units =  self.hardware_code.split("-")[3].split("X")[1].split("Y")[0]
            self.y_units =  self.hardware_code.split("-")[3].split("Y")[1].split("P")[0]

            self.x_cavity_spacing_1 =  self.hardware_code.split("-")[4].split("X")[1].split("Y")[0]
            self.y_cavity_spacing_1 =  self.hardware_code.split("-")[4].split("X")[1].split("Y")[1]
            self.x_cavity_units_1 =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[0]
            self.y_cavity_units_1 =  self.hardware_code.split("-")[5].split("X")[1].split("Y")[1]
            self.x_offset_1  =  self.hardware_code.split("-")[6].split("XO")[1].split("YO")[0]
            self.y_offset_1  =  self.hardware_code.split("-")[6].split("YO")[1]
            self.x_cavity_dimensions_1 =  self.hardware_code.split("-")[7].split("X")[1].split("Y")[0]
            self.y_cavity_dimensions_1 =  self.hardware_code.split("-")[7].split("X")[1].split("Y")[1].split("Z")[0]
            self.z_cavity_dimensions_1 =  self.hardware_code.split("-")[7].split("X")[1].split("Y")[1].split("Z")[1]

            if (self.hardware_code.split("-")[8].count("S") == 1):
                self.cavity_type_1 = '"S"'
            elif (self.hardware_code.split("-")[8].count("C") == 1):
                self.cavity_type_1 = '"C"'
            else:
                self.errors.append("The cavity code present within the provided system-code is invalid.")
                pass

            self.x_cavity_spacing_2 =  self.hardware_code.split("-")[9].split("X")[1].split("Y")[0]
            self.y_cavity_spacing_2 =  self.hardware_code.split("-")[9].split("X")[1].split("Y")[1]
            self.x_cavity_units_2 =  self.hardware_code.split("-")[10].split("X")[1].split("Y")[0]
            self.y_cavity_units_2 =  self.hardware_code.split("-")[10].split("X")[1].split("Y")[1]
            self.x_offset_2  =  self.hardware_code.split("-")[11].split("XO")[1].split("YO")[0]
            self.y_offset_2  =  self.hardware_code.split("-")[11].split("YO")[1]
            self.x_cavity_dimensions_2 =  self.hardware_code.split("-")[12].split("X")[1].split("Y")[0]
            self.y_cavity_dimensions_2 =  self.hardware_code.split("-")[12].split("X")[1].split("Y")[1].split("Z")[0]
            self.z_cavity_dimensions_2 =  self.hardware_code.split("-")[12].split("X")[1].split("Y")[1].split("Z")[1]

            if (self.hardware_code.split("-")[13].count("S") == 1):
                self.cavity_type_2 = '"S"'
            elif (self.hardware_code.split("-")[13].count("C") == 1):
                self.cavity_type_2 = '"C"'
            else:
                self.errors.append("The cavity code present within the provided system-code is invalid.")
                pass

            """Format Inputs"""

            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.shaft_radius = self.shaft_radius.replace("P", ".", 1)
            self.x_cavity_spacing_1 = self.x_cavity_spacing_1.replace("P", ".", 1)
            self.y_cavity_spacing_1 = self.y_cavity_spacing_1.replace("P", ".", 1)
            self.x_offset_1 = self.x_offset_1.replace("P", ".", 1)
            self.y_offset_1 = self.y_offset_1.replace("P", ".", 1)
            self.x_cavity_dimensions_1 = self.x_cavity_dimensions_1.replace("P", ".", 1)
            self.y_cavity_dimensions_1 = self.y_cavity_dimensions_1.replace("P", ".", 1)
            self.z_cavity_dimensions_1 = self.z_cavity_dimensions_1.replace("P", ".", 1)
            self.x_cavity_spacing_2 = self.x_cavity_spacing_2.replace("P", ".", 1)
            self.y_cavity_spacing_2 = self.y_cavity_spacing_2.replace("P", ".", 1)
            self.x_offset_2 = self.x_offset_2.replace("P", ".", 1)
            self.y_offset_2 = self.y_offset_2.replace("P", ".", 1)
            self.x_cavity_dimensions_2 = self.x_cavity_dimensions_2.replace("P", ".", 1)
            self.y_cavity_dimensions_2 = self.y_cavity_dimensions_2.replace("P", ".", 1)
            self.z_cavity_dimensions_2 = self.z_cavity_dimensions_2.replace("P", ".", 1)

            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            ''' Execute write scad functions.   '''

            self.CUBX0177_SPANS2()
            self.scad_file.close()
            
        
        if (self.type_code=="AXLE"): # Square axle.

            # Example SPAN hardware code : "CUBX0177-AXLE-B25SR2P5-A22"
            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0].replace("P", ".", 1) # Block length.
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1].replace("P", ".", 1) # Shaft radius.
            self.axle_blocks = self.hardware_code.split("-")[3].split("A")[1].replace("P", ".", 1) # Axle blocks.

            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''
            
            ''' Testing that code is parsed correctly.'''
            
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("    axle_blocks = "+self.axle_blocks+" blocks")
            print("")
            
            ''' Execute write scad functions.   '''

            self.CUBX0177_AXLE()
            self.scad_file.close()
            
        if (self.type_code=="FYAD"): # Flywheel adapter.

            # Example SPAN hardware code : "CUBX0177-FYAD-B25SR2P5".
            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0] # Block length.
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1].replace("P", ".", 1) # Shaft radius.

            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''

            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.threshaft_radiusad_starts = self.shaft_radius.replace("P", ".", 1)

            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("")
            
            ''' Execute write scad functions.   '''

            self.CUBX0177_FYAD()
            self.scad_file.close()
            
        if (self.type_code=="AXAD"): # Axle adapter.

            # Example AXAD hardware code : "CUBX0177-FYAD-B7P5SR2P22-PP32"
            self.block_unit_length = self.hardware_code.split("-")[2].split("B")[1].split("SR")[0] # Block length.
            self.shaft_radius = self.hardware_code.split("-")[2].split("B")[1].split("SR")[1].replace("P", ".", 1) # Shaft radius.
            self.padding =  "0."+self.hardware_code.split("-")[3].split("PP")[1]
            ''' Rules logic goes here ; conditions of which parameters combinations can exist.   '''
            
            self.block_unit_length = self.block_unit_length.replace("P", ".", 1)
            self.threshaft_radiusad_starts = self.shaft_radius.replace("P", ".", 1)
            
            ''' Testing that code is parsed correctly.'''
            print("")
            print("    Parameterization Information")
            print("")
            print("    famliy_code = "+self.family_code)
            print("    type_code = "+self.type_code)
            print("    block_unit_length = "+self.block_unit_length+"mm")
            print("    shaft_radius = "+self.shaft_radius+"mm")
            print("    padding = "+self.padding+"mm")
            print("")
            
            ''' Execute write scad functions.   '''

            self.CUBX0177_AXAD()
            self.scad_file.close()


    def CUBX0177_BPAN(self): # CUBX0177-BPAN-B25SR2P5-X8Y8PP2-RT-SX25Y25-X8Y8-XO3YO5-X20Y20Z10-S
        self.scad_file.write('CUBX0177_BPAN( '+self.block_unit_length+', '+self.shaft_radius+', '+self.x_units+', '+self.y_units+', '+self.padding+', '+self.orientation+', '+self.teeth+', '+self.x_cavity_spacing+', '+self.y_cavity_spacing+', '+self.x_cavity_units+', '+self.y_cavity_units+', '+self.x_offset+', '+self.y_offset+', '+self.x_cavity_dimensions+', '+self.y_cavity_dimensions+', '+self.z_cavity_dimensions+', '+self.cavity_type+');\n')
   
    def CUBX0177_BPANS2(self): # CUBX0177-BPAN-B25SR2P5-X8Y8PP2-RT-SX25Y25-X8Y8-XO3YO5-X20Y20Z10-S-SX25Y25-X8Y8-XO3YO5-X20Y20Z10-S
        self.scad_file.write('CUBX0177_BPANS2( '+self.block_unit_length+', '+self.shaft_radius+', '+self.x_units+', '+self.y_units+', '+self.padding+', '+self.orientation+', '+self.teeth+', '+self.x_cavity_spacing_1+', '+self.x_cavity_spacing_1+', '+self.x_cavity_units_1+', '+self.y_cavity_units_1+', '+self.x_offset_1+', '+self.y_offset_1+', '+self.x_cavity_dimensions_1+', '+self.y_cavity_dimensions_1+', '+self.z_cavity_dimensions_1+', '+self.cavity_type_1+', '+self.x_cavity_spacing_2+', '+self.x_cavity_spacing_2+', '+self.x_cavity_units_2+', '+self.y_cavity_units_2+', '+self.x_offset_2+', '+self.y_offset_2+', '+self.x_cavity_dimensions_2+', '+self.y_cavity_dimensions_2+', '+self.z_cavity_dimensions_2+', '+self.cavity_type_2+');\n')

    # CUBX0177_SPAN(block_length, shaft_radius, x_units, y_units, x_spacing, y_spacing, x_cavity_units, y_cavity_units, x_offset, y_offset, x_cavity_dimensions, y_cavity_dimensions, z_cavity_dimensions, cavity_type)
    def CUBX0177_SPAN(self):
        self.scad_file.write('CUBX0177_SPAN( block_length = '+self.block_unit_length+', shaft_radius = '+self.shaft_radius+', x_units = '+self.x_units+', y_units = '+self.y_units+', x_cavity_spacing = '+self.x_cavity_spacing+', y_cavity_spacing = '+self.y_cavity_spacing+', x_cavity_units = '+self.x_cavity_units+', y_cavity_units = '+self.y_cavity_units+', x_offset = '+self.x_offset+', y_offset = '+self.y_offset+', x_cavity_dimensions = '+self.x_cavity_dimensions+', y_cavity_dimensions = '+self.y_cavity_dimensions+', z_cavity_dimensions = '+self.z_cavity_dimensions+', cavity_type = '+self.cavity_type+' );\n')
    # X8Y8-XO3YO5-X20Y20Z10-S-SX25Y25-X8Y8-XO3YO5-X20Y20Z10-S
    # CUBX0177_SPANS2( block_length = 7.5, shaft_radius = 2.18, 6, 6, 0, 0, 1, 1, 0, 0, 19, 19, 0.75, "C", 0, 0, 1, 1, 0, 0, 12, 12, 10, "S"); 
    # CUBX0177_SPANS2(block_length, shaft_radius, x_units, y_units, x_cavity_spacing_1, y_cavity_spacing_1, x_cavity_units_1, y_cavity_units_1, x_offset_1, y_offset_1, x_cavity_dimensions_1, y_cavity_dimensions_1, z_cavity_dimensions_1, cavity_type_1, x_cavity_spacing_2, y_cavity_spacing_2, x_cavity_units_2, y_cavity_units_2, x_offset_2, y_offset_2, x_cavity_dimensions_2, y_cavity_dimensions_2, z_cavity_dimensions_2, cavity_type_2)
    # Example SPANS2 hardware code : "CUBX0177-SPANS2-B7P5SR2P22-X8Y8-SX0Y0-X1Y1-XO0YO0-X19Y19Z0P75-C-SX0Y0-X1Y1-XO0YO0-X12Y12Z1P10-S"
    def CUBX0177_SPANS2(self):
        self.scad_file.write('CUBX0177_SPANS2( block_length = '+self.block_unit_length+', shaft_radius = '+self.shaft_radius+', x_units = '+self.x_units+', y_units = '+self.y_units+', x_cavity_spacing_1 = '+self.x_cavity_spacing_1+', y_cavity_spacing_1 = '+self.y_cavity_spacing_1+', x_cavity_units_1 = '+self.x_cavity_units_1+', y_cavity_units_1 = '+self.y_cavity_units_1+', x_offset_1 = '+self.x_offset_1+', y_offset_1 = '+self.y_offset_1+', x_cavity_dimensions_1 = '+self.x_cavity_dimensions_1+', y_cavity_dimensions_1 = '+self.y_cavity_dimensions_1+', z_cavity_dimensions_1 = '+self.z_cavity_dimensions_1+', cavity_type_1 = '+self.cavity_type_1+', x_cavity_spacing_2 = '+self.x_cavity_spacing_2+', y_cavity_spacing_2 = '+self.y_cavity_spacing_2+', x_cavity_units_2 = '+self.x_cavity_units_2+', y_cavity_units_2 = '+self.y_cavity_units_2+', x_offset_2 = '+self.x_offset_2+', y_offset_2 = '+self.y_offset_2+', x_cavity_dimensions_2 = '+self.x_cavity_dimensions_2+', y_cavity_dimensions_2 = '+self.y_cavity_dimensions_2+', z_cavity_dimensions_2 = '+self.z_cavity_dimensions_2+', cavity_type_2 = '+self.cavity_type_2+' );\n')


    def CUBX0177_FYAD(self):
        self.scad_file.write('CUBX0177_FYAD( '+self.block_unit_length+', '+self.shaft_radius+' );\n')

    def CUBX0177_AXAD(self):
        self.scad_file.write('CUBX0177_AXAD( '+self.block_unit_length+', '+self.shaft_radius+', '+self.padding+' );\n')
    
    def CUBX0177_AXLE(self):
        self.scad_file.write('CUBX0177_AXLE( '+self.block_unit_length+', '+self.shaft_radius+', '+self.axle_blocks+' );\n')

class CUBX0177_encoding:
    
    def __init__(self):
        self.type_code
        self.system_code
    
    def encode_session(self):

        #type_code = SelectionBranch("Select CUBX0177 type:")
        #type_code.options.append(["1", "BP", "Box Panel", CUBX0177_BP_input.run])
        #type_code.options.append(["2", "SP", "Simple Panel", exit])
        #type_code.options.append(["3", "SQ", "Square Axle", exit])
        #type_code.options.append(["4", "AA", "Axle Adapter", exit])
        #type_code.options.append(["4", "FA", "Flywheel Adapter", exit])
        #type_code.options.append(["b", "Back", "Navigate to previous menu.", exit])
        
        if (self.type_code=="BPAN"): # Box panel.
            pass
        elif (self.type_code=="SPAN"): # Simple panel.
            pass
        elif (self.type_code=="AXLE"): # Square axle.
            pass
        elif (self.type_code=="FYAD"): # Flywheel adapter.
            pass
        elif (self.type_code=="AXAD"): # Axle adapter.
            pass
        else:
            pass


#!/usr/bin/env python3

from setuptools import find_packages, setup

setup(
    name='mupy',
    version='0.1.12',
    author='Michael Cranston Ryan',
    package_data={'': ['*.scad']},
    include_package_data=True,
    author_email='spacetime.engineer@gmail.com',
    url='http://pypi.python.org/pypi/mupy/',
    license='LICENSE.txt',
    description='Manufacturing Utility',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    classifiers=[
        'Intended Audience :: Education',
        'Intended Audience :: Manufacturing',
        'Intended Audience :: Science/Research',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: BSD License',  
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Other', # Openscad programming language in particular.
    ]
)



#!/usr/bin/env python3
class SelectionBranch:

    # Run at object creation.
    def __init__(self, question):

        self.question = question
        self.options = []

    def pretty_print_options(self):
        for option in self.options: # For each option stored in the object.
            print("  "+option[0]+"  -  "+option[1]+" : "+option[2])

    def run(self):
        mu_symbol = '\u00B5'
        print("\n"+mu_symbol+":! "+self.question+"\n")
        if len(self.options) > 0:
            self.pretty_print_options() # Pretty print options.
            print("")

        user_input = input("  > ") # Takes in user input.
        print("") # Prints a newline.
        for option in self.options: # For each option stored in the object.
            if user_input == option[0]: # For whichever option is selected run its counterpart routine.
                if len(option) == 5: # If the function that is passed needs an accompanying input.
                    option[3](option[4]) # Execute the function (4th element) with the input passed into it (5th element).
                elif len(option) == 4:
                    option[3]() # Execute routine.
                else:
                    pass





#!/usr/bin/env python3

class AssemblyEpoch:
    """ This class stores $t values for assembly animations ; specifically are used to break hirearchies into chunks of equivilent periods of time that automate the otherwise difficylt process"""
    
    def __init__(self, epoch_start, epoch_end):
        self.epoch_start = epoch_start
        self.epoch_end = epoch_end


#!/usr/bin/python3

"""
This class is one of the worst parts of this whole project. 
The reason for it's existence was that there needed to be a place to make function calls for various scad libraries.
It sucks because certification implies based on this structure that we need to inject a new directory for each family code.
At first this seems reasonable but when you consider automating certification to the point where you can make a new family for a trivial assembly, it would be hard to program so many more directories.
I Think there needs to exist a 3rd part database or online marketplace or archive where the code is stored.
"""

class Decode:
    """ This class creates objects which represent a decoding of a knonw hardware code. This object contains a name and hrdware code mostly but handles the hardware library importing routines which provide the system the tools to build the intended hardware element."""

    def __init__(self, hardware_code, directory):
            """Run at object creation."""

            self.hardware_code = hardware_code
            self.family_code = self.hardware_code.split("-")[0]
            self.family_code_valid = True
            
            ''' Checks which family the hardware code belogs to. '''

            family_code = self.hardware_code.split("-")[0]

            if family_code == "CUBX0177":
                from .lib.CUBX0177.cubx0177 import CUBX0177
                CUBX0177(self.hardware_code, directory)
            elif family_code == "CUBX0012":
                #CUBX0012(self.hardware_code, directory)
                pass
            elif family_code == "CUBX0006":
                from .lib.CUBX0006.cubx0006 import CUBX0006
                CUBX0006(self.hardware_code, directory)
                pass
            elif family_code == "CYLX0006":
                from .lib.CYLX0006.cylx0006 import CYLX0006
                CYLX0006(self.hardware_code, directory)
                pass
            elif family_code == "CRSPGR022":
                from .lib.CRSPGR022.crspgr022 import CRSPGR022
                CRSPGR022(self.hardware_code, directory)
                pass
            elif family_code == "CUSBX0177":
                from .lib.CUSBX0177.cusbx0177 import CUSBX0177
                self.digital_twin = CUSBX0177(self.hardware_code, directory)
                self.assembly = self.digital_twin.assembly
            elif family_code == "BOLTX0004":
                from .lib.BOLTX0004.boltx0004 import BOLTX0004
                BOLTX0004(self.hardware_code, directory)
            else:
                self.family_code_valid = False